--::::::::::
--bit.spc
--::::::::::
-- ***************************************************************
-- *                                                             *
-- *  BIT_FUNCTIONS                                              *  SPEC
-- *                                                             *
-- ***************************************************************
package Bit_Functions is
--| Purpose
--|   This package allows the Ada programmer to manipulate the bits
--|   within an object of type INTEGER.  The bits are numbers from
--|   the right to the left, starting with number zero.
--| 
--|           +------------------------+
--|           +  15 14 13 ...  3 2 1 0 !
--|           +------------------------+
--| 
--|      In each routine, the number of bits being manipulated
--|      is NBITS.  START_AT identifies the right most bit of NBITS field.
--| 
--|      e.g.
--|           ...  6 5 4 3 2 1 0
--|                    X X X         nbits = 3
--|                                  start_at = 2
--| 
--| Initialization Exceptions (none)
--| Notes
--|    Not all MIL-HDBK-1804 PDL annotations are
--| used in this package due to its simplicity.
--|    No exceptions are raised by this package.
--|
--| Modifications
--| Author: Freeman Moore, TI

  -- ..................................................................
  -- .                                                                .
  -- .  BIT_FUNCTIONS.BIT_EXTRACT                                     .  SPEC
  -- .                                                                .
  -- ..................................................................
  function Bit_Extract (Item, Start_At, Nbits : INTEGER) return INTEGER;
  --| Purpose
  --| Return the bit field extracted from ITEM, as a signed integer.

  -- ..................................................................
  -- .                                                                .
  -- .  BIT_FUNCTIONS.UBIT_EXTRACT                                    .  SPEC
  -- .                                                                .
  -- ..................................................................
  function Ubit_Extract (Item, Start_At, Nbits : INTEGER) return INTEGER;
  --| Purpose
  --| Return the bit field extracted from ITEM, unsigned integer.

  -- ..................................................................
  -- .                                                                .
  -- .  BIT_FUNCTIONS.BIT_INSERT                                      .  SPEC
  -- .                                                                .
  -- ..................................................................
  function Bit_Insert (This_Item, Nbits, Into_Item, Start_At : INTEGER)
      return INTEGER;
  --| Purpose
  --| Insert NBITS from THIS_ITEM into the object INTO_ITEM,
  --| with the rightmost bit identified by START_AT.

  -- ..................................................................
  -- .                                                                .
  -- .  BIT_FUNCTIONS.BIT_REMOVE                                      .  SPEC
  -- .                                                                .
  -- ..................................................................
  function Bit_Remove (Item, Start_At, Nbits : INTEGER) return INTEGER;
  --| Purpose
  --| BIT_REMOVE will zero out NBITS of ITEM at position START_AT.

  -- ..................................................................
  -- .                                                                .
  -- .  BIT_FUNCTIONS.SHIFT_LEFT                                      .  SPEC
  -- .                                                                .
  -- ..................................................................
  function Shift_Left (Item, Nbits : INTEGER) return INTEGER;
  --| Purpose
  --| Return ITEM shifted left by NBITS.

  -- ..................................................................
  -- .                                                                .
  -- .  BIT_FUNCTIONS.SHIFT_RIGHT                                     .  SPEC
  -- .                                                                .
  -- ..................................................................
  function Shift_Right (Item, Nbits : INTEGER) return INTEGER;
  --| Purpose
  --| Return ITEM shifted right by NBITS.

  -- ..................................................................
  -- .                                                                .
  -- .  BIT_FUNCTIONS.BIT_AND                                         .  SPEC
  -- .                                                                .
  -- ..................................................................
  function Bit_AND (Word1, Word2 : INTEGER) return INTEGER;
  --| Purpose
  --| Return the AND of the two objects.

  -- ..................................................................
  -- .                                                                .
  -- .  BIT_FUNCTIONS.BIT_OR                                          .  SPEC
  -- .                                                                .
  -- ..................................................................
  function Bit_OR (Word1, Word2 : INTEGER) return INTEGER;
  --| Purpose
  --| Return the OR of the two objects.

  -- ..................................................................
  -- .                                                                .
  -- .  BIT_FUNCTIONS.BIT_MASK                                        .  SPEC
  -- .                                                                .
  -- ..................................................................
  function Bit_Mask (Nbits : INTEGER) return INTEGER;
  --| Purpose
  --| Return an object with NBITS of one bits, right justified.

end Bit_Functions;
