--::::::::::
--cisc.spc
--::::::::::
-- *******************************************
-- *                                         *
-- *  CASE_INSENSITIVE_STRING_COMPARISON     *  SPEC
-- *                                         *
-- *******************************************
package Case_Insensitive_String_Comparison is
--| Purpose
--| This package provides a complete set of comparison functions on strings
--| where case is NOT important ("a" = "A").
--|
--| Initialization Exceptions (none)
--| Notes (none)
--|
--| Modifications
--| Programmer: Michael Gordon

  -- .................................................
  -- .                                               .
  -- .  CASE_INSENSITIVE_STRING_COMPARISION.TOUPPER  .  SPEC
  -- .                                               .
  -- .................................................
  function ToUpper (C: CHARACTER) return CHARACTER;
  --| Purpose
  --|          If C is in 'a'..'z' return the corresponding upper case
  --| character.  Otherwise, return C.  This is implemented by a table
  --| lookup for speed.
  --|
  --| Exceptions (none)
  --| Notes (none)

  -- .................................................
  -- .                                               .
  -- .  CASE_INSENSITIVE_STRING_COMPARISION.UPCASE   .  SPEC
  -- .                                               .
  -- .................................................
  procedure UpCase (S: in out STRING);
  --| Purpose
  --|        Convert all characters in S to upper case.
  --|
  --| Exceptions (none)
  --| Notes (none)
    pragma inline(UpCase);

  -- .................................................
  -- .                                               .
  -- .  CASE_INSENSITIVE_STRING_COMPARISION.UPCASE   .  SPEC
  -- .                                               .
  -- .................................................
  function UpCase (S: STRING) return STRING;
  --| Purpose
  --|          Make a copy of S, convert all lower case characters to upper
  --| case and return the copy.
  --|
  --| Exceptions (none)
  --| Notes (none)

  -- .................................................
  -- .                                               .
  -- .  CASE_INSENSITIVE_STRING_COMPARISION.TOLOWER  .  SPEC
  -- .                                               .
  -- .................................................
  function ToLower (C: CHARACTER) return CHARACTER;
  --| Purpose
  --|          If C is in 'A'..'Z' return the corresponding lower case
  --| character.  Otherwise, return C.  This is implemented by a table
  --| lookup for speed.
  --|
  --| Exceptions (none)
  --| Notes (none)

  -- .................................................
  -- .                                               .
  -- .  CASE_INSENSITIVE_STRING_COMPARISION.DOWNCASE .  SPEC
  -- .                                               .
  -- .................................................
  procedure DownCase (S: in out STRING);
  --| Purpose
  --|          Convert all characters in S to lower case.
  --|
  --| Exceptions (none)
  --| Notes (none)
    pragma inline(DownCase);

  -- .................................................
  -- .                                               .
  -- .  CASE_INSENSITIVE_STRING_COMPARISION.DOWNCASE .  SPEC
  -- .                                               .
  -- .................................................
  function DownCase (S: STRING) return STRING;
  --| Purpose
  --|          Make a copy of S, convert all lower case characters to lower
  --| case and return the copy.
  --|
  --| Exceptions (none)
  --| Notes (none)

  -- .................................................
  -- .                                               .
  -- .  CASE_INSENSITIVE_STRING_COMPARISION.COMPARE  .  SPEC
  -- .                                               .
  -- .................................................
  function Compare (P, Q: STRING) return INTEGER;
  --| Purpose
  --|          Return an integer less than zero if P < Q, zero if P = Q, and
  --| an integer greater than zero if P > Q.
  --|
  --| Exceptions (none)
  --| Notes (none)

  -- .................................................
  -- .                                               .
  -- .  CASE_INSENSITIVE_STRING_COMPARISION.EQUAL    .  SPEC
  -- .                                               .
  -- .................................................
  function Equal (P, Q: STRING) return BOOLEAN;
  --| Purpose
  --|        Return TRUE iff P = Q.
  --|
  --| Exceptions (none)
  --| Notes (none)

  -- .................................................
  -- .                                               .
  -- .  CASE_INSENSITIVE_STRING_COMPARISION.LESS     .  SPEC
  -- .                                               .
  -- .................................................
  function Less (P, Q: STRING) return BOOLEAN;
  --| Purpose
  --|        Return TRUE iff P < Q.
  --|
  --| Exceptions (none)
  --| Notes (none)

  -- ......................................................
  -- .                                                    .
  -- .  CASE_INSENSITIVE_STRING_COMPARISION.LESS_OR_EQUAL .  SPEC
  -- .                                                    .
  -- ......................................................
  function Less_or_Equal (P, Q: STRING) return BOOLEAN;
  --| Purpose
  --|        Return TRUE iff P <= Q.
  --|
  --| Exceptions (none)
  --| Notes (none)

  -- .................................................
  -- .                                               .
  -- .  CASE_INSENSITIVE_STRING_COMPARISION.GREATER  .  SPEC
  -- .                                               .
  -- .................................................
  function Greater (P, Q: STRING) return BOOLEAN;
  --| Purpose
  --|        Return TRUE iff P > Q.
  --|
  --| Exceptions (none)
  --| Notes (none)

  -- ..........................................................
  -- .                                                        .
  -- .  CASE_INSENSITIVE_STRING_COMPARISION.GREATER_OR_EQUAL  .  SPEC
  -- .                                                        .
  -- ..........................................................
  function Greater_or_Equal (P, Q: STRING) return BOOLEAN;
  --| Purpose
  --|        Return TRUE iff P >= Q.
  --|
  --| Exceptions (none)
  --| Notes (none)

private

    pragma inline (Equal, Less, Less_or_Equal, Greater, Greater_or_Equal);
    pragma inline (ToUpper, ToLower);

end Case_Insensitive_String_Comparison;
