--::::::::::
--cli.spc
--::::::::::
-- **************************************
-- *                                    *
-- * CLI (Command Line Interface)       * SPEC
-- *                                    *
-- **************************************
package CLI is

--| Purpose
--|   CLI is a package which implements a Command
--| Line Interface.  It mirrors the UNIX/C
--| command line interface, providing an argument
--| count and the arguments themselves.
--|
--| Initialization Exceptions (none)
--|
--| Notes
--|   Compiler limit on string length and dynamic memory.
--|   INITIALIZE must be called once, and only once, during
--| the execution of the main Ada proc.
--|
--| Modifications
--|  2/25/88  Richard Conn    Initial Version
--|  5/12/89  Richard Conn    Review and Upgrade
--|  4/11/90  Richard Conn    MIL-HDBK-1804 Annotations and
--|                           Meridian Ada Interface Added
   
  -- ...................................
  -- .                                 .
  -- . CLI.INITIALIZE                  . SPEC
  -- .                                 .
  -- ...................................
  procedure Initialize (Program_Name        : in STRING;
                        Command_Line_Prompt : in STRING);
  --| Purpose
  --|   Initialize this package.  This routine must be called
  --| before any other routines or objects are called or referenced.
  --|
  --| Exceptions (none)
  --|
  --| Notes
  --|   CALL THIS PROCEDURE ONLY ONE TIME

  -- ...................................
  -- .                                 .
  -- . CLI.ARGC (Argument Count)       . SPEC
  -- .                                 .
  -- ...................................
  function ArgC return NATURAL;
  --| Purpose
  --|   Return the number (1 to N) of command line arguments.
  --| ARGC is at least 1 because the name of the program or
  --| process is always ARGV(0).
  --|
  --| Exceptions (none)
  --| Notes (none)

  -- ...................................
  -- .                                 .
  -- . CLI.ARGV (Argument Value)       . SPEC
  -- .                                 .
  -- ...................................
  function ArgV (Index : in NATURAL) return STRING;
  --| Purpose
  --|   Return the INDEXth (0 <= INDEX < ARGC) command line
  --| argument.  Example: if ARGC = 1, ARGV(0) is the only
  --| valid argument string.  ARGV(0) is always the name of
  --| the program or process.
  --|
  --| Exceptions
  --|   INVALID_INDEX     raised if Index >= ARGC
  --|
  --| Notes (none)

  INVALID_INDEX    : exception;
  UNEXPECTED_ERROR : exception;  -- raised anytime
   
end CLI;
