--::::::::::
--cssc.spc
--::::::::::
-- ********************************************************
-- *                                                      *
-- *  Case_Sensitive_String_Comparison                    *  SPEC
-- *                                                      *
-- ********************************************************
package Case_Sensitive_String_Comparison is
--| Purpose
--| This package provides a complete set of comparison
--| functions on strings where case is important ("a" /= "A").
--| In most cases these have the same effect as the Ada
--| predefined operators.  However, using this package
--| makes it easier to substitute case-insensitive comparison
--| later.
--|
--| Initialization Exceptions (none)
--| Notes
--|    No exceptions are raised by any method, so the MIL-HDBK-1804
--| annotation requirements are reduced.
--|
--| Modifications
--| Author: Michael Gordon, Intermetrics

  -- ...................................................
  -- .                                                 .
  -- .  Case_Sensitive_String_Comparison.Compare       .  SPEC
  -- .                                                 .
  -- ...................................................
  function Compare (P, Q: STRING) return INTEGER;
  --| Purpose
  --| Return an integer less than zero if P < Q, zero if
  --| P = Q, and an integer greater than zero if P > Q.

  -- ...................................................
  -- .                                                 .
  -- .  Case_Sensitive_String_Comparison.Equal         .  SPEC
  -- .                                                 .
  -- ...................................................
  function Equal (P, Q: STRING) return BOOLEAN;
  --| Purpose
  --| Return True iff P = Q.

  -- ...................................................
  -- .                                                 .
  -- .  Case_Sensitive_String_Comparison.Less          .  SPEC
  -- .                                                 .
  -- ...................................................
  function Less (P, Q: STRING) return BOOLEAN;
  --| Purpose
  --| Return True iff P < Q.

  -- ...................................................
  -- .                                                 .
  -- .  Case_Sensitive_String_Comparison.Less_or_Equal .  SPEC
  -- .                                                 .
  -- ...................................................
  function Less_or_Equal (P, Q: STRING) return BOOLEAN;
  --| Purpose
  --| Return True iff P <= Q.

  -- ...................................................
  -- .                                                 .
  -- .  Case_Sensitive_String_Comparison.Greater       .  SPEC
  -- .                                                 .
  -- ...................................................
  function Greater (P, Q: STRING) return BOOLEAN;
  --| Purpose
  --| Return True iff P > Q.

  -- ......................................................
  -- .                                                    .
  -- .  Case_Sensitive_String_Comparison.Greater_or_Equal .  SPEC
  -- .                                                    .
  -- ......................................................
  function Greater_or_Equal (P, Q: STRING) return BOOLEAN;
  --| Purpose
  --| Return True iff P >= Q.

private
    pragma Inline (equal, less, less_or_equal, greater,
                   greater_or_equal);
end Case_Sensitive_String_Comparison;
