--::::::::::
--in.spc
--::::::::::
-- **********************************
-- *                                *
-- *  Input_File                    *  SPEC
-- *                                *
-- **********************************
package Input_File is
--| Purpose
--| Input_File implements an abstract data type of an input file.
--| Input_File offers an abstraction that can be made more efficient
--| by not using Text_IO (and having its associated overhead imposed)
--| if possible,
--|
--| Initialization Exceptions (none)
--| Notes (none)
--|
--| Modifications
--| 08/16/89  Rick Conn    Initial Version

  type FILE_TYPE is
    private;

  Cannot_Open_Input_File
    : exception;
  Read_Error
    : exception;

  -- ..................................
  -- .                                .
  -- .  Input_File.Open               .  SPEC
  -- .                                .
  -- ..................................
  procedure Open
    ( Id             : in out FILE_TYPE;
      File_Name      : in STRING );
  --| Purpose
  --| Open an existing FILE_TYPE object.
  --|
  --| Exceptions
  --|   Cannot_Open_Input_File
  --|
  --| Notes (none)

  -- ..................................
  -- .                                .
  -- .  Input_File.Get_Line           .  SPEC
  -- .                                .
  -- ..................................
  procedure Get_Line
    ( Id             : in out FILE_TYPE;
      Item           : out STRING;
      Last           : out NATURAL );
  --| Purpose
  --| Get_Line reads an Item to the FILE_TYPE object.
  --|
  --| Exceptions
  --|   Read_Error
  --|
  --| Notes (none)

  -- ..................................
  -- .                                .
  -- .  Input_File.End_Of_File        .  SPEC
  -- .                                .
  -- ..................................
  function End_Of_File
    ( Id             : in FILE_TYPE )
      return BOOLEAN;
  --| Purpose
  --| End_Of_File returns TRUE if the FILE_TYPE object is empty or
  --| no more data is in it.
  --|
  --| Exceptions
  --|   Read_Error
  --|
  --| Notes (none)

  -- ..................................
  -- .                                .
  -- .  Input_File.Close              .  SPEC
  -- .                                .
  -- ..................................
  procedure Close
    ( Id             : in out FILE_TYPE );
  --| Purpose
  --| Close closes input from the FILE_TYPE object.
  --|
  --| Exceptions (none)
  --| Notes (none)

private -- Input_File
  type FILE_OBJECT;
  type FILE_TYPE is
    access FILE_OBJECT;

end Input_File;
