--::::::::::
--logical.spc
--::::::::::
-- ***************************************************************
-- *                                                             *
-- *  LOGICAL                                                    *  SPEC
-- *                                                             *
-- ***************************************************************
package Logical is
--| Purpose
--| LOGICAL provides bit-level manipulation on INTEGER objects.
--|
--| Initialization Exceptions (none)
--| Notes
--|    Not all MIL-HDBK-1804 PDL annotations are
--| used in this package due to its simplicity.
--|    No exceptions are raised by this package.
--|
--| Modifications
--| Author: Joseph Orost, Concurrent Computer Corporation

  -- ..................................................................
  -- .                                                                .
  -- .  LOGICAL.ROTATE                                                .  SPEC
  -- .                                                                .
  -- ..................................................................
  function Rotate (Arg, Count : INTEGER) return INTEGER;
  --| Purpose
  --| Return arg rotated count bits.  
  --| If count < 0, rotate is to the right,
  --| else, rotate is to the left.

  -- ..................................................................
  -- .                                                                .
  -- .  LOGICAL.SHIFT                                                 .  SPEC
  -- .                                                                .
  -- ..................................................................
  function Shift (Arg, Count : INTEGER) return INTEGER;
  --| Purpose
  --| Return arg logically shifted count bits.
  --| Bits shifted out either end are lost.
  --| If count < 0, shift is to the right,
  --| else, shift is to the left

  -- ..................................................................
  -- .                                                                .
  -- .  LOGICAL."xor"                                                 .  SPEC
  -- .                                                                .
  -- ..................................................................
  function "xor" (Left, Right : INTEGER) return INTEGER;
  --| Purpose
  --| Return left XOR right.

  -- ..................................................................
  -- .                                                                .
  -- .  LOGICAL."and"                                                 .  SPEC
  -- .                                                                .
  -- ..................................................................
  function "and" (Left, Right : INTEGER) return INTEGER;
  --| Purpose
  --| Return left AND right.

  -- ..................................................................
  -- .                                                                .
  -- .  LOGICAL."or"                                                  .  SPEC
  -- .                                                                .
  -- ..................................................................
  function "or" (Left, Right : INTEGER) return INTEGER;
  --| Purpose
  --| Return left OR right.

  -- ..................................................................
  -- .                                                                .
  -- .  LOGICAL."not"                                                 .  SPEC
  -- .                                                                .
  -- ..................................................................
  function "not" (Right : INTEGER) return INTEGER;
  --| Purpose
  --| Return NOT right.

end Logical;
