--::::::::::
--lparse.spc
--::::::::::
-- *****************************************************
-- *                                                   *
-- *  LINE_PARSER                                      *  SPEC
-- *                                                   *
-- *****************************************************
package Line_Parser is
--| Purpose
--| Line_Parser parses strings in a manner similar to ARGC/ARGV
--| under UNIX.  Function ARGC returns a count of the number of
--| tokens in the string and function ARGV returns each token
--| as a separate substring.
--|
--| Initialization Exceptions (none)
--| Notes (none)
--|
--| Modifications
--| Author:  Richard Conn
   
  -- .................................................
  -- .                                               .
  -- .  LINE_PARSER.INITIALIZE                       .  SPEC
  -- .                                               .
  -- .................................................
   procedure Initialize (Item : in STRING);
  --| Purpose
  --| Initialize this package.  This routine MUST be called
  --| before any other routines.
  --|
  --| Exceptions (none)
  --| Notes (none)
   
  -- ....................................................
  -- .                                                  .
  -- .  LINE_PARSER.ARGC                                .  SPEC
  -- .                                                  .
  -- ....................................................
   function ArgC return NATURAL;
  --| Purpose
  --| Return the number of tokens in the string.
  --|
  --| Exceptions (none)
  --| Notes (none)
   
  -- ...................................................
  -- .                                                 .
  -- .  LINE_PARSER.ARGV                               .  SPEC
  -- .                                                 .
  -- ...................................................
  function ArgV (Index : in NATURAL) return STRING;
  --| Purpose
  --| Return the Nth token in the string (the first token is
  --| numbered 0).  Valid values for INDEX are from 0 to
  --| ARGC-1.
  --|
  --| Exceptions
  --|   INVALID_INDEX is raised if INDEX > ARGC-1
  --|
  --| Notes (none)
   
   INVALID_INDEX    : exception;
   UNEXPECTED_ERROR : exception;
   
end Line_Parser;
