--::::::::::
--stringer.spc
--::::::::::
-- *****************************************************************
-- *                                                               *
-- *  STRING_MANIPULATOR                                           *  SPEC
-- *                                                               *
-- *****************************************************************
package String_Manipulator is
--| Purpose   
--| STRING_MANIPULATOR provides a few routines
--| for storing string values into different
--| sizes of strings.
--|
--| Initialization Exceptions (none)
--| Notes (none)
--|
--| Modifications
--| Author: Richard Conn
   
-- Exceptions
  STRING_OVERFLOW : exception; -- raised by GUARDED_LOAD
   
  -- .............................................................
  -- .                                                           .
  -- .  STRING_MANIPULATOR.LOAD                                  .  SPEC
  -- .                                                           .
  -- .............................................................
  procedure Load (From           : in STRING;
                  To             : out STRING;
                  Fill_Character : in CHARACTER := ' ');
  --| Purpose
  --| LOAD places the string FROM into the first part of the
  --| string TO, filling the rest with FILL_CHARACTER; if the string
  --| FROM is longer than the string TO, the string FROM is truncated
  --| into TO without warning
  --|
  --| Exceptions (none)
  --| Notes (none)
   
  -- .............................................................
  -- .                                                           .
  -- .  STRING_MANIPULATOR.LOAD                                  .  SPEC
  -- .                                                           .
  -- .............................................................
  procedure Load (From           : in STRING;
                  To             : out STRING;
                  Last           : out NATURAL;
                  Fill_Character : in CHARACTER := ' ');
  --| Purpose
  --| LOAD places the string FROM into the first part of the
  --| string TO, filling the rest with FILL_CHARACTER; if the string
  --| FROM is longer than the string TO, the string FROM is truncated
  --| into TO without warning
  --|
  --| Exceptions (none)
  --| Notes (none)
   
  -- .............................................................
  -- .                                                           .
  -- .  STRING_MANIPULATOR.GUARDED_LOAD                          .  SPEC
  -- .                                                           .
  -- .............................................................
  procedure Guarded_Load (From           : in STRING;
                          To             : out STRING;
                          Fill_Character : in CHARACTER := ' ');
  --| Purpose
  --| GUARDED_LOAD places the string FROM into the first part of the
  --| string TO, filling the rest with FILL_CHARACTER; if the string
  --| FROM is longer than the string TO, the exception STRING_OVERFLOW is
  --| raised
  --|
  --| Exceptions
  --|   STRING_OVERFLOW
  --|
  --| Notes (none)
   
  -- .............................................................
  -- .                                                           .
  -- .  STRING_MANIPULATOR.GUARDED_LOAD                          .  SPEC
  -- .                                                           .
  -- .............................................................
  procedure Guarded_Load (From           : in STRING;
                          To             : out STRING;
                          Last           : out NATURAL;
                          Fill_Character : in CHARACTER := ' ');
  --| Purpose
  --| GUARDED_LOAD places the string FROM into the first part of the
  --| string TO, filling the rest with FILL_CHARACTER; if the string
  --| FROM is longer than the string TO, the exception STRING_OVERFLOW is
  --| raised
  --|
  --| Exceptions
  --|   STRING_OVERFLOW
  --|
  --| Notes (none)
   
  -- .............................................................
  -- .                                                           .
  -- .  STRING_MANIPULATOR.FILL                                  .  SPEC
  -- .                                                           .
  -- .............................................................
  procedure Fill (What      : out STRING;
                  With_Item : in CHARACTER := ' ');
  --| Purpose
  --| FILL fills the string WHAT with the indicated WITH_ITEM
  --|
  --| Exceptions (none)
  --| Notes (none)
   
  -- .............................................................
  -- .                                                           .
  -- .  STRING_MANIPULATOR.IS_FILLED                             .  SPEC
  -- .                                                           .
  -- .............................................................
  function Is_Filled (What      : in STRING;
                      With_Item : in CHARACTER := ' ') return BOOLEAN;
  --| Purpose
  --| IS_FILLED returns TRUE if the string WHAT contains only the
  --| character WITH_ITEM; IS_FILLED returns FALSE otherwise
  --|
  --| Exceptions (none)
  --| Notes (none)
   
end String_Manipulator;
