--::::::::::
--types.spc
--::::::::::
-- ***************************************************
-- *                                                 *
-- *  CS_Parts_Types                                 *  SPEC
-- *                                                 *
-- ***************************************************
package CS_Parts_Types is
--| Purpose
--| Provide common type definitions for items in CS_Parts
--| and useful conversion utilities.
--|
--| Initialization Exceptions (none)
--| Notes
--|    Not all MIL-HDBK-1804 PDL annotations are
--| used in this package due to its simplicity.
--|    No exceptions are raised in this package.
--|
--| Modifications
--| 07/15/90  Rick Conn  Initial Design and Code

  type BYTE is range 16#0# .. 16#FF#;
  for BYTE'SIZE use 8;

  -- ...................................................
  -- .                                                 .
  -- .  CS_Parts_Types.Convert                         .  SPEC
  -- .                                                 .
  -- ...................................................
  function Convert (Item : in CHARACTER) return BYTE;
  --| Purpose
  --| Convert a CHARACTER into a BYTE.

  -- ...................................................
  -- .                                                 .
  -- .  CS_Parts_Types.Convert                         .  SPEC
  -- .                                                 .
  -- ...................................................
  function Convert (Item : in INTEGER) return BYTE;
  --| Purpose
  --| Convert an INTEGER into a BYTE.  If the
  --| INTEGER is greater than 255, only the low-order
  --| BYTE is converted.

  -- ...................................................
  -- .                                                 .
  -- .  CS_Parts_Types.Convert                         .  SPEC
  -- .                                                 .
  -- ...................................................
  function Convert (Item : in BYTE) return CHARACTER;
  --| Purpose
  --| Convert a BYTE into a CHARACTER.  If the most
  --| significant bit of the BYTE is set, it is cleared
  --| as the CHARACTER.

  -- ...................................................
  -- .                                                 .
  -- .  CS_Parts_Types.Convert                         .  SPEC
  -- .                                                 .
  -- ...................................................
  function Convert (Item : in BYTE) return INTEGER;
  --| Purpose
  --| Convert a BYTE into a INTEGER.

end CS_Parts_Types;
