
with Text_Io;

package body Source_Scanner is

  subtype Lines is String (1 .. 80);

  Upto             : Integer; -- col within row, or row in file
  Line             : Lines;
  Next_Label       : Integer := 100;
  Current          : Character;
  Scanner_File     : Text_Io.File_Type;
  Last             : Integer;
  Still_More_Stuff : Boolean := True;


  function Capitals_Of (S : String) return String is
    Temp : String (S'range) := S;
  begin
    for A in Temp'range loop
      case Temp (A) is
        when 'a' .. 'z' =>
	  Temp (A) := Character'Val (Character'Pos (Temp (A)) - 32);

        when others =>
	  null;
      end case;
    end loop;

    return Temp;
  end Capitals_Of;


  function String_Of (Message      : String;
                      Until_Length : Integer) return String is
  begin
    if Message'Length >= Until_Length then
      return Message (Message'First .. Message'First + Until_Length - 1);
    else
      return Message & (Message'Length + 1 .. Until_Length => ' ');
    end if;
  end String_Of;


  procedure Get_Next_Token is

    type Scanner_States is
	 (Start,             Ident_String,      Ident_Under_Score,
	  Digit_String,      Digit_Under_Score, Decimal_Point,
	  Decimal_Part,      Literal_String,    Quotes,
	  Multi_Delims,      Accept_Token,      Commenting);

    State : Scanner_States := Start;

    Lex_Error                  : exception;
    End_Of_Stuff               : exception;
    Couldnt_Get_Next_Character : exception;
    Inconsistent_Double_Op     : exception;


    procedure Condense is
    begin
      Token80 (Token_Upto) := Current;
      Token_Upto := Token_Upto + 1;
    end Condense;


    procedure Get_Next_Line is
    begin
      Line := (others => ' ');
      Text_Io.Get_Line (Scanner_File, Line, Last);
      Upto := 0;
      Current := ' ';
    exception
      when Text_Io.End_Error => 
	Text_Io.Close (Scanner_File);
	Current := ' ';
	Line := (others => ' ');
	Upto := Line'Last;
	Still_More_Stuff := False;
    end Get_Next_Line;


    procedure Get_Next_Character is
    begin
      if Upto = 0 then
	Upto := Line'First;
      else
	Upto := Upto + 1;

	if State = Start then
	  while Line (Upto) in Character'Val (1) .. ' ' loop
	    Upto := Upto + 1;
	  end loop;
	end if;
      end if;

      Current := Line (Upto);
    exception
      when Program_Error =>  raise;

      when Constraint_Error => 
	if Still_More_Stuff then
	  Get_Next_Line;
	else
	  raise End_Of_Stuff;
	end if;

      when others =>  raise Couldnt_Get_Next_Character;
    end Get_Next_Character;
  begin
    Token_Upto := Columns'First;

    loop
      case State is
	when Start => 
	  case Current is
	    when 'a' .. 'z' | 'A' .. 'Z' => 
	      State := Ident_String;

	    when '0' .. '9' => 
	      State := Digit_String;

	    when '"' => 
	      State := Literal_String;
	      Get_Next_Character;

	    when '-' | '<' | '>' | '=' | '*' | ':' | '/' | '.' | '+' => 
	      State := Multi_Delims;

	    when Character'Val (1) .. ' ' => 
	      Get_Next_Character;

	    when others =>  -- is a single delimiter
	      Condense;
	      Input_Symbol := Op;
	      State := Accept_Token;
	      Get_Next_Character;
	  end case;

	when Ident_String => 
	  case Current is
	    when 'a' .. 'z' | 'A' .. 'Z' | '0' .. '9' => 
	      Condense;
	      Get_Next_Character;

	    when Scanner_Under_Score => 
	      Condense;
	      Get_Next_Character;
	      State := Ident_Under_Score;

	    when others => 
	      if Scanner_All_Capitals then
		Token80 (1 .. Token_Upto) :=
		  Capitals_Of (Token80 (1 .. Token_Upto));
	      end if;

	      Input_Symbol := Id;
	      State := Accept_Token;
	  end case;

	when Ident_Under_Score => 
	  case Current is
	    when 'a' .. 'z' | 'A' .. 'Z' | '0' .. '9' => 
	      State := Ident_String;

	    when others => 
	      raise Lex_Error;
	  end case;

	when Digit_String => 
	  case Current is
	    when '0' .. '9' => 
	      Condense;
	      Get_Next_Character;

	    when Scanner_Under_Score => 
	      Condense;
	      Get_Next_Character;
	      State := Digit_Under_Score;

	    when '.' => 
	      Get_Next_Character;
	      State := Decimal_Point;

	    when others => 
	      Input_Symbol := Literal;
	      State := Accept_Token;
	  end case;

	when Digit_Under_Score => 
	  case Current is
	    when '0' .. '9' => 
	      State := Digit_String;

	    when others => 
	      raise Lex_Error;
	  end case;

	when Decimal_Point => 
	  case Current is
	    when '.' => 
	      State := Accept_Token;
	      Input_Symbol := Literal;
	      Line := String_Of ("..", Line'Length);
	      Upto := 0;
	      Get_Next_Character;

	    when '0' .. '9' => 
	      Token80 (Token_Upto) := '.';
	      Token_Upto := Token_Upto + 1;
	      State := Decimal_Part;

	    when others => 
	      raise Lex_Error;
	  end case;

	when Decimal_Part => 
	  case Current is
	    when '0' .. '9' => 
	      Condense;
	      Get_Next_Character;

	    when others => 
	      Input_Symbol := Literal;
	      State := Accept_Token;
	  end case;

	when Literal_String => 
	  case Current is
	    when Scanner_Quote => 
	      Get_Next_Character;
	      State := Quotes;

	    when others => 
	      Condense;
	      Get_Next_Character;
	  end case;

	when Quotes => 
	  case Current is
	    when Scanner_Quote => 
	      Condense;
	      Get_Next_Character;
	      State := Literal_String;

	    when others => 
	      Input_Symbol := Literal;
	      State := Accept_Token;
	  end case;

	when Multi_Delims => 
	  case Current is
	    when '-' => 
	      Get_Next_Character;

	      if Current = '-' then
		State := Commenting;
	      else
		Token80 (1) := '-';
		Token_Upto := 2;
		State := Accept_Token;
	      end if;

	    when '>' | '/' => 
	      Condense;
	      Get_Next_Character;

	      if Current = '=' then
		Condense;
		Get_Next_Character;
	      end if;

	      State := Accept_Token;

	    when '<' => 
	      Condense;
	      Get_Next_Character;

	      if (Current = '>') or (Current = '=') then
		Condense;
		Get_Next_Character;
	      end if;

	      State := Accept_Token;

	    when ':' => 
	      Condense;
	      Get_Next_Character;

	      case Current is
		when '=' => 
		  Condense;
		  Get_Next_Character;

		when ':' => 
		  Condense;
		  Get_Next_Character;

		  if Current = '=' then
		    Condense;
		    Get_Next_Character;
		  end if;

		when others =>  null;
	      end case;

	      State := Accept_Token;

	    when '=' => 
	      Condense;
	      Get_Next_Character;

	      if Current = '>' then
		Condense;
		Get_Next_Character;
	      end if;

	      State := Accept_Token;

	    when '.' => 
	      Condense;
	      Get_Next_Character;

	      if Current = '.' then
		Condense;
		Get_Next_Character;
	      end if;

	      State := Accept_Token;

	    when '+' => 
	      Condense;
	      Get_Next_Character;

	      if Current = '*' then
		Condense;
		Get_Next_Character;
	      end if;

	      State := Accept_Token;

	    when '*' => 
	      Condense;
	      Get_Next_Character;

	      case Current is
		when '*' | '+' => 
		  Condense;
		  Get_Next_Character;

		when others =>  null;
	      end case;

	      State := Accept_Token;

	    when others =>  raise Inconsistent_Double_Op;
	  end case;

	  Input_Symbol := Op;

	when Commenting => 
	  State := Start;
	  Get_Next_Line;

	when Accept_Token => 
	  Token_Upto := Token_Upto - 1;
	  exit;
      end case;
    end loop;
  exception
    when Scanner_End_Of_File =>  raise;

    when Lex_Error => 
      Text_Io.Put_Line ("***");
      Text_Io.Put_Line ("->" & Current & "<- cannot follow =>" &
			Token80 (1 .. Token_Upto - 1) & "<=");
      raise;

    when End_Of_Stuff => 
      if Input_Symbol = End_Of_Input then
	raise Scanner_End_Of_File;
      else
	Token80 (1) := '|';
	Token_Upto := 1;
	Input_Symbol := Symbols'(End_Of_Input);
      end if;

    when Constraint_Error =>  -- Probably token_upto = 0.
      Token_Upto := 1;
      Token80 (1) := ' ';
      Input_Symbol := Literal;
  end Get_Next_Token;


  procedure Closing_Procedure_Test is
  begin
    Text_Io.Close (Scanner_File);
  exception
    when others =>  null;
  end Closing_Procedure_Test;


  procedure Initialize_Scanner (File_Name : String) is
    This_File : Text_Io.File_Type;
  begin
    Closing_Procedure_Test;
    Text_Io.Open (Scanner_File, Text_Io.In_File, File_Name);
    Line := (others => ' ');
    Text_Io.Get_Line (Scanner_File, Line, Last);
    Upto := 0;
    Current := ' ';
    Still_More_Stuff := True;
    Input_Symbol := Literal;
    Token80 := (others => ' ');
    Token_Upto := 1;
  exception
    when Text_Io.End_Error => 
      Text_Io.Close (Scanner_File);
      Current := ' ';
      Line := (others => ' ');
      Upto := Line'Last;
      Still_More_Stuff := False;
  end Initialize_Scanner;

end Source_Scanner;
