with Unchecked_Deallocation;

package body Generic_B_Plus_Tree is

   --  *************************************************************************************
   --  **  This software is part of the Clemson University Computer Science Department's  **
   --  **  Ada Software Repository, and is copyrighted (C) 1989 by Clemson University.    **
   --  **  Permission to copy without fee all or part of this software is granted,        **
   --  **  provided that the copies are not made or distributed for direct commercial     **
   --  **  advantage, and that this copyright notice is not deleted or modified.  To      **
   --  **  copy otherwise, or to republish, requires a fee and/or specific permission.    **
   --  **  >> All bug reporters receive a free updated copy once the bug's corrected! <<  ** 
   --  **  E-mail to: cpscada@citron.cs.clemson.edu or ...!gatech!hubcap!citron!cpscada.  **
   --  *************************************************************************************

   ----------------------------------------------------------------------------------------------------------------------------
   type Internal_Node (Index_Node : Boolean);
   ----------------------------------------------------------------------------------------------------------------------------
   type Internal_Node_Pointer is access Internal_Node;       
   ----------------------------------------------------------------------------------------------------------------------------
   Maximum_Number_Of_Subtrees_Per_Node : constant := 3;    -- This can be any odd number >= 3...
                                                           -- Unfortunately, due to limitations of Ada,
                                                           --   this cannot be made into a generic parameter.
   ----------------------------------------------------------------------------------------------------------------------------
   Minimum_Number_Of_Subtrees_Per_Node : constant := (Maximum_Number_Of_Subtrees_Per_Node/ 2) + 1;
   ----------------------------------------------------------------------------------------------------------------------------
   Minimum_Subtree_Number : constant := Minimum_Number_Of_Subtrees_Per_Node - 1;
   ----------------------------------------------------------------------------------------------------------------------------
   Maximum_Subtree_Number : constant := Maximum_Number_Of_Subtrees_Per_Node - 1;
   ----------------------------------------------------------------------------------------------------------------------------
   type Subtrees is range 0..Maximum_Subtree_Number;
   ----------------------------------------------------------------------------------------------------------------------------
   type Array_Of_Subtrees is array (Subtrees) of Internal_Node_Pointer;
   ----------------------------------------------------------------------------------------------------------------------------
   Maximum_Number_Of_Keys_Per_Node : constant := Maximum_Number_Of_Subtrees_Per_Node - 1;
   ----------------------------------------------------------------------------------------------------------------------------
   type Keys is range 1..Maximum_Number_Of_Keys_Per_Node;
   ----------------------------------------------------------------------------------------------------------------------------
   type Key_Pointer is access Key_Type;                     
   ----------------------------------------------------------------------------------------------------------------------------
   type Array_Of_Keys is array (Keys) of Key_Pointer;
   ----------------------------------------------------------------------------------------------------------------------------
   type Internal_Node (Index_Node : Boolean) is 
      record
         case Index_Node is
            when True  => Maximum_Subtree_Index : Subtrees;
                          Key                   : Array_Of_Keys;
                          Subtree               : Array_Of_Subtrees;

            when False => Preceding_Leaf        : Internal_Node_Pointer := null;
                          Following_Leaf        : Internal_Node_Pointer := null;
                          Key_Value             : Key_Pointer;
                          Item_Container        : Non_Key_Item_Container;
         end case;
      end record;
   ----------------------------------------------------------------------------------------------------------------------------
   type B_Plus_Tree_Descriptor is 
      record
         Root_Node             : Internal_Node_Pointer;
         Current_Leaf          : Internal_Node_Pointer;
         Minimum_Key           : Key_Pointer;
         Number_Of_Keys_Stored : Natural;
      end record;
   ----------------------------------------------------------------------------------------------------------------------------
   type Non_Key_Item_Pointer is access Non_Key_Item_Type;
   ----------------------------------------------------------------------------------------------------------------------------
   type Type_Of_Deletion is (Key, Non_Key_Object);
   ----------------------------------------------------------------------------------------------------------------------------
   Null_Node_Pointer : Internal_Node_Pointer := null;    -- acceptable as an {in out} parameter...
   Null_Key_Pointer  : Key_Pointer           := null;    -- acceptable as an {in out} parameter...
   ----------------------------------------------------------------------------------------------------------------------------
   function Empty (Targeted_B_Plus_Tree : in B_Plus_Tree) return Boolean is

   begin   -- function Empty
      return (Targeted_B_Plus_Tree = null);
   end Empty;
   ----------------------------------------------------------------------------------------------------------------------------
   function Number_Of_Keys_Stored (Targeted_B_Plus_Tree : in B_Plus_Tree) return Natural is

   begin   -- function Number_Of_Keys_Stored
      if (Targeted_B_Plus_Tree = null) then
         return 0;
      else
         return Targeted_B_Plus_Tree.Number_Of_Keys_Stored;
      end if;
   end Number_Of_Keys_Stored;
   ----------------------------------------------------------------------------------------------------------------------------
   procedure Exchange (First_Key_Pointer  : in out Key_Pointer;
                       Second_Key_Pointer : in out Key_Pointer) is

      Temp_Key_Pointer : Key_Pointer := First_Key_Pointer;

   begin   -- procedure Exchange
      First_Key_Pointer  := Second_Key_Pointer;
      Second_Key_Pointer := Temp_Key_Pointer;
   end Exchange;
   ----------------------------------------------------------------------------------------------------------------------------
   procedure Exchange (First_Internal_Node_Pointer  : in out Internal_Node_Pointer;
                       Second_Internal_Node_Pointer : in out Internal_Node_Pointer) is

      Temp_Internal_Node_Pointer : Internal_Node_Pointer := First_Internal_Node_Pointer;

   begin   -- procedure Exchange
      First_Internal_Node_Pointer  := Second_Internal_Node_Pointer;
      Second_Internal_Node_Pointer := Temp_Internal_Node_Pointer;
   end Exchange;
   ----------------------------------------------------------------------------------------------------------------------------
   procedure Exchange (First_B_Plus_Tree  : in out B_Plus_Tree;
                       Second_B_Plus_Tree : in out B_Plus_Tree) is

      Temporary_B_Plus_Tree : B_Plus_Tree := First_B_Plus_Tree;

   begin   -- procedure Exchange
      First_B_Plus_Tree := Second_B_Plus_Tree;
      Second_B_Plus_Tree := Temporary_B_Plus_Tree;
   end Exchange;
   ----------------------------------------------------------------------------------------------------------------------------
   function Determine_Path_Of_Descent (Targeted_Index_Node : in Internal_Node_Pointer;
                                       Key_Value           : in Key_Type                ) return Subtrees is

      Path_Number : Subtrees := 0;

   begin   -- function Determine_Path_Of_Descent
      while (Path_Number < Maximum_Subtree_Number)
       and then ( Targeted_Index_Node.Key (Keys'Val (Subtrees'Pos (Path_Number) +1) )  /= null ) 
       and then not Less_Than (Key_Value, Targeted_Index_Node.Key (Keys'Val (Subtrees'Pos(Path_Number)+1)).all) loop
         Path_Number := Path_Number + 1;
      end loop;
      return Path_Number;
   end Determine_Path_Of_Descent;
   ----------------------------------------------------------------------------------------------------------------------------
   procedure Left_Shift (Node_Being_Shifted     : in out Internal_Node_Pointer;
                         Minimum_Key_In_Subtree : in out Key_Pointer;
                         Leftmost_Shift_Point   : in     Subtrees;
                         Rightmost_Shift_Point  : in     Subtrees               ) is

      -- Assumption: Leftmost_Shift_Point > 0...

   begin   -- procedure Left_Shift
      for Subtree_Number in Leftmost_Shift_Point..Rightmost_Shift_Point loop
         Node_Being_Shifted.Subtree (Subtree_Number - 1) := Node_Being_Shifted.Subtree (Subtree_Number);
         if (Subtree_Number > 1) then
            Node_Being_Shifted.Key ( Keys'Val (Subtrees'Pos (Subtree_Number - 1) ) )
               := Node_Being_Shifted.Key ( Keys'Val (Subtrees'Pos (Subtree_Number) ) );
         else
            Minimum_Key_In_Subtree := Node_Being_Shifted.Key ( Keys'Val (Subtrees'Pos (Subtree_Number) ) );
         end if;
      end loop;
   end Left_Shift;
   ----------------------------------------------------------------------------------------------------------------------------
   procedure Right_Shift (Node_Being_Shifted     : in out Internal_Node_Pointer;
                          Minimum_Key_In_Subtree : in out Key_Pointer;
                          Leftmost_Shift_Point   : in     Subtrees;
                          Rightmost_Shift_Point  : in     Subtrees               ) is

      -- Assumption: Rightmost_Shift_Point < Maximum_Subtree_Number...

   begin   -- procedure Right_Shift
      for Subtree_Number in reverse Leftmost_Shift_Point..Rightmost_Shift_Point loop
         Node_Being_Shifted.Subtree (Subtree_Number + 1) := Node_Being_Shifted.Subtree (Subtree_Number);
         if (Subtree_Number > 0) then
            Node_Being_Shifted.Key ( Keys'Val (Subtrees'Pos (Subtree_Number + 1) ) )
               := Node_Being_Shifted.Key ( Keys'Val (Subtrees'Pos (Subtree_Number) ) );
         else
            Node_Being_Shifted.Key (1) := Minimum_Key_In_Subtree;
         end if;
      end loop;
   end Right_Shift;
   ----------------------------------------------------------------------------------------------------------------------------
   procedure Insert_Extra_Subtree (Targeted_Index_Node          : in out Internal_Node_Pointer;
                                   Minimum_Key_In_Subtree       : in out Key_Pointer;
                                   Extra_Subtree                : in out Internal_Node_Pointer;
                                   Minimum_Key_In_Extra_Subtree : in out Key_Pointer           ) is

      -- Assumption: Targeted_Index_Node.Maximum_Subtree_Index < Maximum_Subtree_Number...

      Path_Of_Descent : Subtrees;
      Insertion_Point : Subtrees;

   begin   -- procedure Insert_Extra_Subtree
      Path_Of_Descent := Determine_Path_Of_Descent (Targeted_Index_Node, Minimum_Key_In_Extra_Subtree.all);
      if (Path_Of_Descent > 0) 
       or else Less_Than (Minimum_Key_In_Subtree.all, Minimum_Key_In_Extra_Subtree.all) then
         Insertion_Point := Path_Of_Descent + 1;
      else
         Insertion_Point := Path_Of_Descent;
      end if;
      Right_Shift (Targeted_Index_Node, Minimum_Key_In_Subtree, Insertion_Point, Targeted_Index_Node.Maximum_Subtree_Index);
      Targeted_Index_Node.Subtree (Insertion_Point) := Extra_Subtree;
      if (Insertion_Point = 0) then
         Minimum_Key_In_Subtree := Minimum_Key_In_Extra_Subtree;
      else
         Targeted_Index_Node.Key ( Keys'Val (Subtrees'Pos (Insertion_Point))) := Minimum_Key_In_Extra_Subtree;
      end if;
      Targeted_Index_Node.Maximum_Subtree_Index := Targeted_Index_Node.Maximum_Subtree_Index + 1;
      Extra_Subtree                := null;
      Minimum_Key_In_Extra_Subtree := null;
   end Insert_Extra_Subtree;
   ----------------------------------------------------------------------------------------------------------------------------
   procedure Delete_Subtree (Targeted_Node          : in out Internal_Node_Pointer;
                             Minimum_Key_In_Subtree : in out Key_Pointer;
                             Targeted_Subtree       : in     Subtrees              ) is

   begin   -- procedure Delete_Subtree
      if (Targeted_Subtree < Targeted_Node.Maximum_Subtree_Index) then
         Left_Shift (Targeted_Node, Minimum_Key_In_Subtree, (Targeted_Subtree+1), Targeted_Node.Maximum_Subtree_Index);
      end if;
      Targeted_Node.Subtree (Targeted_Node.Maximum_Subtree_Index) := null;
      if (Targeted_Node.Maximum_Subtree_Index = 0) then
         Minimum_Key_In_Subtree := null;
      else
         Targeted_Node.Key ( Keys'Val ( Subtrees'Pos (Targeted_Node.Maximum_Subtree_Index) ) ) := null;
         Targeted_Node.Maximum_Subtree_Index := Targeted_Node.Maximum_Subtree_Index - 1;
      end if;
   end Delete_Subtree;
   ----------------------------------------------------------------------------------------------------------------------------
   procedure Insert_Item (Targeted_B_Plus_Tree : in out B_Plus_Tree;
                          Key_Value            : in     Key_Type;
                          Non_Key_Information  : in     Non_Key_Item_Type) is


      New_Root                     : Internal_Node_Pointer;
      Extra_Subtree                : Internal_Node_Pointer := null;
      Minimum_Key_In_Extra_Subtree : Key_Pointer           := null;

      procedure Generate_New_Leaf (Pointer_To_Preceding_Leaf : in     Internal_Node_Pointer;
                                   Pointer_To_New_Leaf       : in out Internal_Node_Pointer;
                                   Pointer_To_Following_Leaf : in     Internal_Node_Pointer;
                                   Value_Of_New_Key          : in     Key_Type               ) is

      begin   -- procedure Generate_New_Leaf
         Pointer_To_New_Leaf := new Internal_Node (Index_Node => False);
         Pointer_To_New_Leaf.Preceding_Leaf := Pointer_To_Preceding_Leaf;
         Pointer_To_New_Leaf.Following_Leaf := Pointer_To_Following_Leaf;
         if (Pointer_To_Preceding_Leaf /= null) then
            Pointer_To_Preceding_Leaf.Following_Leaf := Pointer_To_New_Leaf;
         end if;
         if (Pointer_To_Following_Leaf /= null) then
            Pointer_To_Following_Leaf.Preceding_Leaf := Pointer_To_New_Leaf;
         end if;
         Pointer_To_New_Leaf.Key_Value := new Key_Type;
         Assign (Pointer_To_New_Leaf.Key_Value.all, Value_Of_New_Key);
      end Generate_New_Leaf;

     
      procedure Create_New_B_Plus_Tree (Targeted_B_Plus_Tree : in out B_Plus_Tree;
                                        Key_Value            : in     Key_Type;
                                        Non_Key_Information  : in     Non_Key_Item_Type) is

      begin   -- procedure Create_New_B_Plus_Tree
         Targeted_B_Plus_Tree := new B_Plus_Tree_Descriptor;
         Targeted_B_Plus_Tree.Root_Node := new Internal_Node (Index_Node => True);
         Generate_New_Leaf (null, Targeted_B_Plus_Tree.Root_Node.Subtree(0), null, Key_Value);
         Insert (Targeted_B_Plus_Tree.Root_Node.Subtree(0).Item_Container, Non_Key_Information);
         Targeted_B_Plus_Tree.Current_Leaf := Targeted_B_Plus_Tree.Root_Node.Subtree(0);
         Targeted_B_Plus_Tree.Minimum_Key := Targeted_B_Plus_Tree.Root_Node.Subtree(0).Key_Value;
         Targeted_B_Plus_Tree.Root_Node.Maximum_Subtree_Index := 0;
         Targeted_B_Plus_Tree.Number_Of_Keys_Stored := 1;
      end Create_New_B_Plus_Tree;


      procedure Insert_Subtree (Left_Sibling                 : in out Internal_Node_Pointer;
                                Minimum_Key_In_Left_Sibling  : in out Key_Pointer;
                                Targeted_Index_Node          : in out Internal_Node_Pointer;
                                Minimum_Key_In_Subtree       : in out Key_Pointer;
                                Right_Sibling                : in out Internal_Node_Pointer;
                                Minimum_Key_In_Right_Sibling : in out Key_Pointer;
                                Extra_Subtree                : in out Internal_Node_Pointer;
                                Minimum_Key_In_Extra_Subtree : in out Key_Pointer            ) is


         Room_In_Left_Sibling : Boolean := ( (Left_Sibling /= null)
                                                and then (Left_Sibling.Index_Node = True)
                                                    and then (Left_Sibling.Maximum_Subtree_Index < Maximum_Subtree_Number) );

         Room_In_Right_Sibling : Boolean := ( (Right_Sibling /= null)
                                                and then (Right_Sibling.Index_Node = True)
                                                    and then (Right_Sibling.Maximum_Subtree_Index < Maximum_Subtree_Number) );

         type Overflow_Preference is (Return_Leftmost_Subtree, Return_Rightmost_Subtree);


         procedure Insert_And_Overflow (Targeted_Index_Node          : in out Internal_Node_Pointer;
                                        Minimum_Key_In_Subtree       : in out Key_Pointer;
                                        Extra_Subtree                : in out Internal_Node_Pointer;
                                        Minimum_Key_In_Extra_Subtree : in out Key_Pointer;
                                        Overflow_Directions          : in     Overflow_Preference   ) is

            Temp_Subtree    : Internal_Node_Pointer;
            Temp_Key        : Key_Pointer;
            Insertion_Point : Subtrees := Determine_Path_Of_Descent (Targeted_Index_Node, Minimum_Key_In_Extra_Subtree.all);

         begin   -- procedure Insert_And_Overflow
            if (Overflow_Directions = Return_Leftmost_Subtree) then
               if (Insertion_Point = 0) then
                  if Less_Than (Minimum_Key_In_Subtree.all, Minimum_Key_In_Extra_Subtree.all) then 
                     Exchange (Targeted_Index_Node.Subtree(0), Extra_Subtree);
                     Exchange (Minimum_Key_In_Extra_Subtree, Minimum_Key_In_Subtree);
                  end if;
               else
                  Temp_Subtree := Targeted_Index_Node.Subtree(0);
                  Temp_Key     := Minimum_Key_In_Subtree;
                  Left_Shift (Targeted_Index_Node, Minimum_Key_In_Subtree, 1, Insertion_Point);
                  Targeted_Index_Node.Subtree (Insertion_Point) := Extra_Subtree;
                  Targeted_Index_Node.Key (Keys'Val(Subtrees'Pos(Insertion_Point))) := Minimum_Key_In_Extra_Subtree;
                  Extra_Subtree                := Temp_Subtree;
                  Minimum_Key_In_Extra_Subtree := Temp_Key;
               end if;
            elsif (Overflow_Directions = Return_Rightmost_Subtree) then
               if (Insertion_Point + 1 >= Maximum_Subtree_Number) then
                  if not Less_Than (Targeted_Index_Node.Key (Maximum_Subtree_Number).all, Minimum_Key_In_Extra_Subtree.all) then
                     Exchange (Extra_Subtree, Targeted_Index_Node.Subtree (Maximum_Subtree_Number) );
                     Exchange (Minimum_Key_In_Extra_Subtree, 
                                     Targeted_Index_Node.Key (Keys'Val (Subtrees'Pos (Maximum_Subtree_Number) ) ) );
                  end if;
               else
                  if not (Insertion_Point = 0) 
                   or else Less_Than (Minimum_Key_In_Subtree.all, Minimum_Key_In_Extra_Subtree.all) then
                     Insertion_Point := Insertion_Point + 1;
                  end if;
                  Temp_Subtree := Targeted_Index_Node.Subtree (Maximum_Subtree_Number);
                  Temp_Key     := Targeted_Index_Node.Key (Maximum_Subtree_Number);
                  Right_Shift (Targeted_Index_Node, Minimum_Key_In_Subtree, Insertion_Point, (Maximum_Subtree_Number - 1));
                  Targeted_Index_Node.Subtree (Insertion_Point) := Extra_Subtree;
                  if (Insertion_Point = 0) then
                     Minimum_Key_In_Subtree := Minimum_Key_In_Extra_Subtree;
                  else
                     Targeted_Index_Node.Key ( Keys'Val ( Subtrees'Pos ( Insertion_Point ) ) ) := Minimum_Key_In_Extra_Subtree;
                  end if;
                  Extra_Subtree                := Temp_Subtree;
                  Minimum_Key_In_Extra_Subtree := Temp_Key;
               end if;
            end if;
         end Insert_And_Overflow;


         procedure Insert_And_Partition (Targeted_Index_Node          : in out Internal_Node_Pointer;
                                         Minimum_Key_In_Subtree       : in out Key_Pointer;
                                         Extra_Subtree                : in out Internal_Node_Pointer;
                                         Minimum_Key_In_Extra_Subtree : in out Key_Pointer            ) is

            Insertion_Point                  : Subtrees;
            New_Extra_Subtree                : Internal_Node_Pointer;
            Minimum_Key_In_New_Extra_Subtree : Key_Pointer;

            procedure Partition (Targeted_Index_Node              : in out Internal_Node_Pointer;
                                 Minimum_Key_In_Subtree           : in out Key_Pointer;
                                 Node_Split_Point                 : in     Subtrees;
                                 New_Extra_Subtree                : in out Internal_Node_Pointer;
                                 Minimum_Key_In_New_Extra_Subtree : in out Key_Pointer             ) is

            begin   -- procedure Partition

               New_Extra_Subtree := new Internal_Node (Index_Node => True);

               for Transferred_Subtree_Index in reverse Node_Split_Point..Maximum_Subtree_Number loop
                  New_Extra_Subtree.Subtree (Transferred_Subtree_Index - Node_Split_Point)
                     := Targeted_Index_Node.Subtree (Transferred_Subtree_Index);
                  Targeted_Index_Node.Subtree (Transferred_Subtree_Index) := null;
                  if (Transferred_Subtree_Index - Node_Split_Point) > 0 then
                     New_Extra_Subtree.Key (Keys'Val ( Subtrees'Pos (Transferred_Subtree_Index - Node_Split_Point)))
                        := Targeted_Index_Node.Key (Keys'Val ( Subtrees'Pos (Transferred_Subtree_Index)));
                  else
                     Minimum_Key_In_New_Extra_Subtree 
                        := Targeted_Index_Node.Key (Keys'Val (Subtrees'Pos (Transferred_Subtree_Index) ) );
                  end if;
                  Targeted_Index_Node.Key (Keys'Val (Subtrees'Pos (Transferred_Subtree_Index))) := null;
               end loop;

               Targeted_Index_Node.Maximum_Subtree_Index := Node_Split_Point - 1;

               New_Extra_Subtree.Maximum_Subtree_Index := Maximum_Subtree_Number - Node_Split_Point;

            end Partition;

         begin     -- procedure Insert_And_Partition

            Insertion_Point := Determine_Path_Of_Descent (Targeted_Index_Node, Minimum_Key_In_Extra_Subtree.all);

            if (Insertion_Point < Minimum_Subtree_Number) then
               Partition (Targeted_Index_Node, Minimum_Key_In_Subtree, Minimum_Subtree_Number, 
                                                                         New_Extra_Subtree, Minimum_Key_In_New_Extra_Subtree);
               Insert_Extra_Subtree (Targeted_Index_Node, Minimum_Key_In_Subtree, Extra_Subtree, Minimum_Key_In_Extra_Subtree);
            else
               Partition (Targeted_Index_Node, Minimum_Key_In_Subtree, Minimum_Subtree_Number + 1, 
                                                                         New_Extra_Subtree, Minimum_Key_In_New_Extra_Subtree);
               Insert_Extra_Subtree (New_Extra_Subtree, Minimum_Key_In_New_Extra_Subtree,
                                                                         Extra_Subtree, Minimum_Key_In_Extra_Subtree);
            end if;

            Extra_Subtree                := New_Extra_Subtree;
            Minimum_Key_In_Extra_Subtree := Minimum_Key_In_New_Extra_Subtree;

         end Insert_And_Partition;


      begin     -- procedure Insert_Subtree
         if (Targeted_Index_Node.Maximum_Subtree_Index < Maximum_Subtree_Number) then
            Insert_Extra_Subtree (Targeted_Index_Node, Minimum_Key_In_Subtree, Extra_Subtree, Minimum_Key_In_Extra_Subtree);
         elsif Room_In_Left_Sibling and not Room_In_Right_Sibling then
            Insert_And_Overflow (Targeted_Index_Node, Minimum_Key_In_Subtree,
                                                 Extra_Subtree, Minimum_Key_In_Extra_Subtree, Return_Leftmost_Subtree);
            Insert_Extra_Subtree (Left_Sibling, Minimum_Key_In_Left_Sibling, Extra_Subtree, Minimum_Key_In_Extra_Subtree);
         elsif Room_In_Right_Sibling then
            Insert_And_Overflow (Targeted_Index_Node, Minimum_Key_In_Subtree,
                                                 Extra_Subtree, Minimum_Key_In_Extra_Subtree, Return_Rightmost_Subtree);
            Insert_Extra_Subtree (Right_Sibling, Minimum_Key_In_Right_Sibling, Extra_Subtree, Minimum_Key_In_Extra_Subtree);
         else
            Insert_And_Partition (Targeted_Index_Node, Minimum_Key_In_Subtree, Extra_Subtree, Minimum_Key_In_Extra_Subtree);
         end if;
      end Insert_Subtree;


      procedure Descend_And_Insert_Leaf (Left_Sibling                 : in     Internal_Node_Pointer;
                                         Target_Node                  : in     Internal_Node_Pointer;
                                         Right_Sibling                : in     Internal_Node_Pointer;
                                         Minimum_Key_In_Subtree       : in out Key_Pointer;
                                         Extra_Subtree                : in out Internal_Node_Pointer;
                                         Minimum_Key_In_Extra_Subtree : in out Key_Pointer           ) is

         Path_Of_Descent : Subtrees;

      begin   -- procedure Descend_And_Insert_Leaf

         if (Target_Node.Index_Node = False) then 

            if Equal (Target_Node.Key_Value.all, Key_Value) then

               Insert (Target_Node.Item_Container, Non_Key_Information);
               Targeted_B_Plus_Tree.Current_Leaf := Target_Node;

            else

               if Less_Than (Key_Value, Target_Node.Key_Value.all) then
                  Generate_New_Leaf (Target_Node.Preceding_Leaf, Extra_Subtree, Target_Node, Key_Value);
               else
                  Generate_New_Leaf (Target_Node, Extra_Subtree, Target_Node.Following_Leaf, Key_Value);
               end if;

               Insert (Extra_Subtree.Item_Container, Non_Key_Information);
               Minimum_Key_In_Extra_Subtree := Extra_Subtree.Key_Value;
               Targeted_B_Plus_Tree.Current_Leaf := Extra_Subtree;

               Targeted_B_Plus_Tree.Number_Of_Keys_Stored := Targeted_B_Plus_Tree.Number_Of_Keys_Stored + 1;

            end if;

         else                         

            Path_Of_Descent := Determine_Path_Of_Descent (Target_Node, Key_Value);

            case Path_Of_Descent is

                when 0 
                   => Descend_And_Insert_Leaf (Null_Node_Pointer, 
                                                 Target_Node.Subtree (0), 
                                                   Target_Node.Subtree (1),
                                                     Minimum_Key_In_Subtree,
                                                       Extra_Subtree, Minimum_Key_In_Extra_Subtree);
                     if (Extra_Subtree /= null) 
                      and (Target_Node.Subtree (Path_Of_Descent).all.Index_Node = True) then
                        Insert_Subtree (Null_Node_Pointer,
                                          Null_Key_Pointer,
                                            Target_Node.Subtree (0),
                                              Minimum_Key_In_Subtree,
                                                Target_Node.Subtree (1),
                                                  Target_Node.Key (1),
                                                    Extra_Subtree, Minimum_Key_In_Extra_Subtree); 
                     end if;

               when 1..(Maximum_Subtree_Number - 1)
                  => Descend_And_Insert_Leaf (Target_Node.Subtree (Path_Of_Descent - 1), 
                                                Target_Node.Subtree (Path_Of_Descent), 
                                                  Target_Node.Subtree (Path_Of_Descent + 1),
                                                    Target_Node.Key ( Keys'Val ( Subtrees'Pos (Path_Of_Descent))),
                                                      Extra_Subtree, Minimum_Key_In_Extra_Subtree);
                     if (Extra_Subtree /= null) 
                      and (Target_Node.Subtree (Path_Of_Descent).Index_Node = True) then
                        if (Path_Of_Descent = 1) then
                           Insert_Subtree (Target_Node.Subtree (0),
                                             Minimum_Key_In_Subtree,
                                               Target_Node.Subtree (1),
                                                 Target_Node.Key (1),
                                                   Target_Node.Subtree (2),
                                                     Target_Node.Key (2),
                                                       Extra_Subtree, Minimum_Key_In_Extra_Subtree); 
                        else
                           Insert_Subtree (Target_Node.Subtree (Path_Of_Descent - 1),
                                             Target_Node.Key (Keys'Val (Subtrees'Pos (Path_Of_Descent - 1))),
                                               Target_Node.Subtree (Path_Of_Descent),
                                                 Target_Node.Key ( Keys'Val ( Subtrees'Pos (Path_Of_Descent))),
                                                   Target_Node.Subtree (Path_Of_Descent + 1),
                                                     Target_Node.Key ( Keys'Val (Subtrees'Pos (Path_Of_Descent-1))),
                                                       Extra_Subtree, Minimum_Key_In_Extra_Subtree); 
                        end if;
                     end if;

               when Maximum_Subtree_Number
                  => Descend_And_Insert_Leaf (Target_Node.Subtree (Maximum_Subtree_Number - 1), 
                                                Target_Node.Subtree (Maximum_Subtree_Number), 
                                                  Null_Node_Pointer,
                                                    Target_Node.Key (Maximum_Subtree_Number),
                                                      Extra_Subtree, Minimum_Key_In_Extra_Subtree);
                     if (Extra_Subtree /= null) 
                      and (Target_Node.Subtree (Path_Of_Descent).Index_Node = True) then
                        Insert_Subtree (Target_Node.Subtree (Maximum_Subtree_Number - 1),
                                          Target_Node.Key (Maximum_Subtree_Number - 1),
                                            Target_Node.Subtree (Maximum_Subtree_Number),
                                              Target_Node.Key (Maximum_Subtree_Number),
                                                Null_Node_Pointer,
                                                  Null_Key_Pointer,
                                                    Extra_Subtree, Minimum_Key_In_Extra_Subtree); 
                     end if;

            end case;

         end if;

      end Descend_And_Insert_Leaf;


   begin    -- procedure Insert_Item

      if (Targeted_B_Plus_Tree = null) then

         Create_New_B_Plus_Tree (Targeted_B_Plus_Tree, Key_Value, Non_Key_Information);

      else

         Descend_And_Insert_Leaf (null, Targeted_B_Plus_Tree.Root_Node, null, 
                                     Targeted_B_Plus_Tree.Minimum_Key, Extra_Subtree, Minimum_Key_In_Extra_Subtree);

         if (Extra_Subtree /= null) then
            Insert_Subtree (Null_Node_Pointer, Null_Key_Pointer, 
                                    Targeted_B_Plus_Tree.Root_Node, Targeted_B_Plus_Tree.Minimum_Key, 
                                              Null_Node_Pointer, Null_Key_Pointer,
                                                       Extra_Subtree, Minimum_Key_In_Extra_Subtree);
         end if;

         if (Extra_Subtree /= null) then
            New_Root := new Internal_Node (Index_Node => True);
            New_Root.Subtree(0) := Targeted_B_Plus_Tree.Root_Node; 
            New_Root.Subtree(1) := Extra_Subtree;
            New_Root.Key(1) := Minimum_Key_In_Extra_Subtree;
            New_Root.Maximum_Subtree_Index := 1;
            Targeted_B_Plus_Tree.Root_Node := New_Root;
         end if;
         
      end if;

   end Insert_Item;
   ----------------------------------------------------------------------------------------------------------------------------
   procedure Destroy is new Unchecked_Deallocation (Key_Type, Key_Pointer);
   ----------------------------------------------------------------------------------------------------------------------------
   procedure Destroy is new Unchecked_Deallocation (Internal_Node, Internal_Node_Pointer);
   ----------------------------------------------------------------------------------------------------------------------------
   procedure Annihilate is new Unchecked_Deallocation (B_Plus_Tree_Descriptor, B_Plus_Tree);
   ----------------------------------------------------------------------------------------------------------------------------
   procedure Destroy_Subtree (Target_Node : in out Internal_Node_Pointer) is

   begin   -- procedure Destroy_Subtree
      if (Target_Node /= null) then 
         if (Target_Node.Index_Node = False) then
            if (Target_Node.Preceding_Leaf /= null) then
               Target_Node.Preceding_Leaf.Following_Leaf := Target_Node.Following_Leaf;
            end if;
            if (Target_Node.Following_Leaf /= null) then
               Target_Node.Following_Leaf.Preceding_Leaf := Target_Node.Preceding_Leaf;
            end if;
            Destroy (Target_Node.Key_Value);
            Destroy_Contents (Target_Node.Item_Container);
            Destroy (Target_Node);
         else
            for Subtree_Number in Subtrees loop
               Destroy_Subtree (Target_Node.Subtree (Subtree_Number));
            end loop;
            Destroy (Target_Node);  
         end if;
      end if;
   end Destroy_Subtree;
   ----------------------------------------------------------------------------------------------------------------------------
   procedure Destroy (Targeted_B_Plus_Tree : in out B_Plus_Tree) is

      -- Destroys all keys and all associated containers, and renders the tree Empty.

   begin   -- procedure Destroy
      if not Empty (Targeted_B_Plus_Tree) then
         Destroy_Subtree (Targeted_B_Plus_Tree.Root_Node);
         Annihilate (Targeted_B_Plus_Tree);
      end if;
   end Destroy;
   ----------------------------------------------------------------------------------------------------------------------------
   procedure Destroy (Targeted_Object : in out Pointer_To_B_Plus_Tree) is

      procedure Annihilate is new Unchecked_Deallocation (B_Plus_Tree, Pointer_To_B_Plus_Tree);

   begin   -- procedure Destroy
      if (Targeted_Object /= null) then
         Destroy (Targeted_Object.all);
         Annihilate (Targeted_Object);
      end if;
   end Destroy;
   ----------------------------------------------------------------------------------------------------------------------------
   procedure Descend_And_Delete (Targeted_B_Plus_Tree   : in out B_Plus_Tree;
                                 Target_Node            : in out Internal_Node_Pointer;
                                 Minimum_Key_In_Subtree : in out Key_Pointer;
                                 Deletion_Type          : in     Type_Of_Deletion;
                                 Target_Key             : in     Key_Type;
                                 Target_Non_Key_Object  : in     Non_Key_Item_Type) is


      Path_Of_Descent   : Subtrees;


      procedure Delete_From_Leaf (Target_Leaf           : in out Internal_Node_Pointer;
                                  Deletion_Type         : in     Type_Of_Deletion;
                                  Target_Key            : in     Key_Type;
                                  Target_Non_Key_Object : in     Non_Key_Item_Type) is

         procedure Delete_Leaf (Targeted_B_Plus_Tree : in out B_Plus_Tree;
                                Target_Leaf          : in out Internal_Node_Pointer) is

         begin   -- procedure Delete_Leaf
            if (Targeted_B_Plus_Tree.Current_Leaf = Target_Leaf) then
               Targeted_B_Plus_Tree.Current_Leaf := null;
            end if;
            if (Targeted_B_Plus_Tree.Minimum_Key = Target_Leaf.Key_Value) then
               if (Target_Leaf.Following_Leaf /= null) then
                  Targeted_B_Plus_Tree.Minimum_Key := Target_Leaf.Following_Leaf.Key_Value;
               else
                  Targeted_B_Plus_Tree.Minimum_Key := null;
               end if;
            end if;
            Destroy_Subtree (Target_Leaf);
            Targeted_B_Plus_Tree.Number_Of_Keys_Stored := Targeted_B_Plus_Tree.Number_Of_Keys_Stored - 1;
         end Delete_Leaf;

      begin   -- procedure Delete_From_Leaf
         if not Equal (Target_Leaf.Key_Value.all, Target_Key) then
            raise Key_Does_Not_Exist_In_This_B_Plus_Tree;
         elsif (Deletion_Type = Non_Key_Object) then
            Delete (Target_Leaf.Item_Container, Target_Non_Key_Object);
            if Empty (Target_Leaf.Item_Container) then
               Delete_Leaf (Targeted_B_Plus_Tree, Target_Leaf);
            end if;
         else   -- (Deletion_Type = Key) 
            Delete_Leaf (Targeted_B_Plus_Tree, Target_Leaf);
         end if;
      end Delete_From_Leaf;


      procedure Redistribute_Subtrees (Target_Node            : in out Internal_Node_Pointer;
                                       Minimum_Key_In_Subtree : in out Key_Pointer;
                                       Path_Of_Descent        : in     Subtrees               ) is

         Needy_Node                   : Internal_Node_Pointer;
         Minimum_Key_In_Needy_Node    : Key_Pointer;
         Left_Sibling                 : Internal_Node_Pointer := null;
         Minimum_Key_In_Left_Sibling  : Key_Pointer           := null;
         Right_Sibling                : Internal_Node_Pointer := null;
         Minimum_Key_In_Right_Sibling : Key_Pointer           := null;
         Extras_In_Left_Sibling       : Boolean;
         Extras_In_Right_Sibling      : Boolean;

      begin   -- procedure Redistribute_Subtrees

         Needy_Node := Target_Node.Subtree (Path_Of_Descent);

         if (Path_Of_Descent = 0) then
            Minimum_Key_In_Needy_Node := Minimum_Key_In_Subtree;
         else
            Minimum_Key_In_Needy_Node := Target_Node.Key ( Keys'Val ( Subtrees'Pos (Path_Of_Descent) ) );
         end if;

         case Path_Of_Descent is

            when 0 => Right_Sibling                := Target_Node.Subtree (1);
                      Minimum_Key_In_Right_Sibling := Target_Node.Key (1);

            when 1 => Left_Sibling                 := Target_Node.Subtree (0);
                      Minimum_Key_In_Left_Sibling  := Minimum_Key_In_Subtree;
                      Right_Sibling                := Target_Node.Subtree (2);
                      Minimum_Key_In_Right_Sibling := Target_Node.Key (2);

            when 2..(Maximum_Subtree_Number - 1) 
                   => Left_Sibling                 := Target_Node.Subtree (Path_Of_Descent - 1);
                      Minimum_Key_In_Left_Sibling  := Target_Node.Key (Keys'Val(Subtrees'Pos(Path_Of_Descent - 1)));
                      Right_Sibling                := Target_Node.Subtree (Path_Of_Descent + 1);
                      Minimum_Key_In_Right_Sibling := Target_Node.Key (Keys'Val(Subtrees'Pos(Path_Of_Descent + 1)));

            when Maximum_Subtree_Number
                   => Left_Sibling                 := Target_Node.Subtree (Maximum_Subtree_Number - 1);
                      Minimum_Key_In_Left_Sibling  := Target_Node.Key (Maximum_Subtree_Number - 1); 

         end case;

         Extras_In_Left_Sibling := ( (Left_Sibling /= null) 
                                        and then (Left_Sibling.Index_Node = True)
                                           and then (Left_Sibling.Maximum_Subtree_Index > Minimum_Subtree_Number));

         Extras_In_Right_Sibling := ( (Right_Sibling /= null) 
                                        and then (Right_Sibling.Index_Node = True)
                                           and then (Right_Sibling.Maximum_Subtree_Index > Minimum_Subtree_Number));

         if Extras_In_Left_Sibling and not Extras_In_Right_Sibling then

            Insert_Extra_Subtree (Needy_Node, Minimum_Key_In_Needy_Node,
                                     Left_Sibling.Subtree (Left_Sibling.Maximum_Subtree_Index),
                                        Left_Sibling.Key (Keys'Val(Subtrees'Pos(Left_Sibling.Maximum_Subtree_Index))));
            Delete_Subtree (Left_Sibling, Minimum_Key_In_Left_Sibling, Left_Sibling.Maximum_Subtree_Index);
            Target_Node.Key ( Keys'Val ( Subtrees'Pos (Path_Of_Descent) ) ) := Minimum_Key_In_Needy_Node;

         elsif Extras_In_Right_Sibling then

            Insert_Extra_Subtree (Needy_Node, Minimum_Key_In_Needy_Node, Right_Sibling.Subtree(0), Minimum_Key_In_Right_Sibling);
            Delete_Subtree (Right_Sibling, Minimum_Key_In_Right_Sibling, 0);
            Target_Node.Key ( Keys'Val ( Subtrees'Pos (Path_Of_Descent + 1) ) ) := Minimum_Key_In_Right_Sibling;

         elsif (Right_Sibling = null) or else (Left_Sibling /= null) then

            Insert_Extra_Subtree (Left_Sibling, Minimum_Key_In_Left_Sibling, Needy_Node.Subtree (0), Minimum_Key_In_Needy_Node);
            for Subtree_Number in 1..Needy_Node.Maximum_Subtree_Index loop
               Insert_Extra_Subtree (Left_Sibling, Minimum_Key_In_Left_Sibling, Needy_Node.Subtree (Subtree_Number), 
                                                              Needy_Node.Key ( Keys'Val ( Subtrees'Pos ( Subtree_Number ) ) ) );
            end loop;
            Destroy (Needy_Node);
            Delete_Subtree (Target_Node, Minimum_Key_In_Subtree, Path_Of_Descent);

         else  

            Insert_Extra_Subtree (Needy_Node, Minimum_Key_In_Needy_Node, Right_Sibling.Subtree (0), Minimum_Key_In_Right_Sibling);
            for Subtree_Number in 1..Right_Sibling.Maximum_Subtree_Index loop
               Insert_Extra_Subtree (Needy_Node, Minimum_Key_In_Needy_Node, Right_Sibling.Subtree (Subtree_Number),
                                                             Right_Sibling.Key ( Keys'Val ( Subtrees'Pos ( Subtree_Number))));
            end loop;
            Destroy (Right_Sibling);
            Delete_Subtree (Target_Node, Minimum_Key_In_Subtree, Path_Of_Descent + 1);

         end if;

      end Redistribute_Subtrees;


   begin   -- procedure Descend_And_Delete

      if (Target_Node.Index_Node = False) then 

         Delete_From_Leaf (Target_Node, Deletion_Type, Target_Key, Target_Non_Key_Object);

      else

         Path_Of_Descent := Determine_Path_Of_Descent (Target_Node, Target_Key);

         if (Path_Of_Descent = 0) then
            Descend_And_Delete ( Targeted_B_Plus_Tree, Target_Node.Subtree (0), 
                                    Minimum_Key_In_Subtree, Deletion_Type, Target_Key, Target_Non_Key_Object );
         else
            Descend_And_Delete ( Targeted_B_Plus_Tree, Target_Node.Subtree (Path_Of_Descent), 
                                   Target_Node.Key ( Keys'Val ( Subtrees'Pos (Path_Of_Descent) ) ),
                                      Deletion_Type, Target_Key, Target_Non_Key_Object                    );
         end if;

         if (Target_Node.Subtree (Path_Of_Descent) = null) then
            Delete_Subtree (Target_Node, Minimum_Key_In_Subtree, Path_Of_Descent);
         elsif (Target_Node.Subtree (Path_Of_Descent).Index_Node = True) 
           and then (Target_Node.Subtree (Path_Of_Descent).Maximum_Subtree_Index < Minimum_Subtree_Number) then
            Redistribute_Subtrees (Target_Node, Minimum_Key_In_Subtree, Path_Of_Descent);
         end if;

      end if;
   end Descend_And_Delete;
   ----------------------------------------------------------------------------------------------------------------------------
   procedure Delete_Key (Targeted_B_Plus_Tree : in out B_Plus_Tree;
                         Search_Key           : in     Key_Type    ) is

      -- Raises Key_Does_Not_Exist_In_This_B_Plus_Tree 
      --   or No_Keys_Currently_Exist_In_This_Empty_B_Plus_Tree, when appropriate...
      --
      -- The Non_Key_Item_Container associated with this key will be emptied via the Destroy_Contents procedure.

      Null_Non_Key_Information : Non_Key_Item_Type;

   begin   -- procedure Delete_Key
      if Empty (Targeted_B_Plus_Tree) then
         raise No_Keys_Currently_Exist_In_This_Empty_B_Plus_Tree;
      else
         Descend_And_Delete (Targeted_B_Plus_Tree, Targeted_B_Plus_Tree.Root_Node, 
                               Targeted_B_Plus_Tree.Minimum_Key, Key, Search_Key, Null_Non_Key_Information);
      end if;
   end Delete_Key;
   ----------------------------------------------------------------------------------------------------------------------------
   procedure Delete_Item (Targeted_B_Plus_Tree : in out B_Plus_Tree;
                          Key_Value            : in     Key_Type;
                          Non_Key_Information  : in     Non_Key_Item_Type) is

      -- Raises Key_Does_Not_Exist_In_This_B_Plus_Tree 
      --   or No_Keys_Currently_Exist_In_This_Empty_B_Plus_Tree, when appropriate...

   begin   -- procedure Delete_Item
      if Empty (Targeted_B_Plus_Tree) then
         raise No_Keys_Currently_Exist_In_This_Empty_B_Plus_Tree;
      else
         Descend_And_Delete (Targeted_B_Plus_Tree, Targeted_B_Plus_Tree.Root_Node, 
                                Targeted_B_Plus_Tree.Minimum_Key, Non_Key_Object, Key_Value, Non_Key_Information);
         Targeted_B_Plus_Tree.Current_Leaf := null;
         if (Targeted_B_Plus_Tree.Root_Node.Maximum_Subtree_Index = 0) then
            if (Targeted_B_Plus_Tree.Root_Node.Subtree(0) /= null) then
               if (Targeted_B_Plus_Tree.Root_Node.Subtree(0).Index_Node = True) then
                  declare
                     Temp_Root : Internal_Node_Pointer := Targeted_B_Plus_Tree.Root_Node.Subtree(0);
                  begin
                     Destroy (Targeted_B_Plus_Tree.Root_Node);
                     Targeted_B_Plus_Tree.Root_Node := Temp_Root;
                  end;
               end if;
            else
               Destroy (Targeted_B_Plus_Tree.Root_Node);
               Destroy (Targeted_B_Plus_Tree);
            end if;
         end if;
      end if;
   end Delete_Item;
   ----------------------------------------------------------------------------------------------------------------------------
   function Get_First_Key (Targeted_B_Plus_Tree : in B_Plus_Tree) return Key_Type is

      -- Raises No_Keys_Currently_Exist_In_This_Empty_B_Plus_Tree when appropriate...

   begin   -- function Get_First_Key
      if Empty (Targeted_B_Plus_Tree) then
         raise No_Keys_Currently_Exist_In_This_Empty_B_Plus_Tree;
      else
         return Targeted_B_Plus_Tree.Minimum_Key.all;
      end if;
   end Get_First_Key;
   ----------------------------------------------------------------------------------------------------------------------------
   function Get_Last_Key (Targeted_B_Plus_Tree : in B_Plus_Tree) return Key_Type is

      -- Raises No_Keys_Currently_Exist_In_This_Empty_B_Plus_Tree when appropriate...

      function Return_Last_Key (This_Subtree : Internal_Node_Pointer) return Key_Type is

      begin   -- function Return_Last_Key
         if (This_Subtree.Index_Node = True) then
            return Return_Last_Key (This_Subtree.Subtree(This_Subtree.Maximum_Subtree_Index));
         else
            return This_Subtree.Key_Value.all;
         end if;
      end Return_Last_Key;

   begin   -- function Get_Last_Key
      if Empty (Targeted_B_Plus_Tree) then
         raise No_Keys_Currently_Exist_In_This_Empty_B_Plus_Tree;
      else
         return Return_Last_Key (Targeted_B_Plus_Tree.Root_Node);
      end if;
   end Get_Last_Key;
   ----------------------------------------------------------------------------------------------------------------------------
   function Get_Leaf_Node (This_B_Plus_Tree : in B_Plus_Tree;
                           Key_Value        : in Key_Type            ) return Internal_Node_Pointer is

      function Find_Leaf_Node (This_Subtree : in Internal_Node_Pointer;
                               Key_Value    : in Key_Type               ) return Internal_Node_Pointer is

         Path_Number : Subtrees := 0;

      begin   -- function Find_Leaf_Node
         if (This_Subtree.Index_Node = True) then
            while (Path_Number < This_Subtree.Maximum_Subtree_Index)
              and then not Less_Than (Key_Value, This_Subtree.Key(Keys'Val(Subtrees'Pos(Path_Number) + 1)).all) loop
               Path_Number := Path_Number + 1;
            end loop;
            return Find_Leaf_Node (This_Subtree.Subtree(Path_Number), Key_Value);
         else
            if Equal (This_Subtree.Key_Value.all, Key_Value) then
               return This_Subtree;
            else
               return null;
            end if;
         end if;
      end Find_Leaf_Node;

   begin   -- function Get_Leaf_Node
      if not Empty (This_B_Plus_Tree) then
         if (This_B_Plus_Tree.Current_Leaf /= null)
           and then Equal (This_B_Plus_Tree.Current_Leaf.Key_Value.all, Key_Value) then
            return This_B_Plus_Tree.Current_Leaf;
         else
            return Find_Leaf_Node (This_B_Plus_Tree.Root_Node, Key_Value);
         end if;
      else
         return null;
      end if;
   end Get_Leaf_Node;
   ----------------------------------------------------------------------------------------------------------------------------
   function Key_Exists (Targeted_B_Plus_Tree : in B_Plus_Tree;
                        Search_Key           : in Key_Type    ) return Boolean is

   begin   -- function Key_Exists
      if not Empty (Targeted_B_Plus_Tree) then
         Targeted_B_Plus_Tree.Current_Leaf := Get_Leaf_Node (Targeted_B_Plus_Tree, Search_Key);
         return (Targeted_B_Plus_Tree.Current_Leaf /= null);
      else
         return False;
      end if;
   end Key_Exists;
   ----------------------------------------------------------------------------------------------------------------------------
   function Get_Item_Container (Targeted_B_Plus_Tree : in B_Plus_Tree;
                                Search_Key           : in Key_Type    ) return Non_Key_Item_Container is

      -- Raises Key_Does_Not_Exist_In_This_B_Plus_Tree 
      --   or No_Keys_Currently_Exist_In_This_Empty_B_Plus_Tree, when appropriate...

   begin   -- function Get_Item_Container
      if Empty (Targeted_B_Plus_Tree) then
         raise No_Keys_Currently_Exist_In_This_Empty_B_Plus_Tree;
      else
         Targeted_B_Plus_Tree.Current_Leaf := Get_Leaf_Node (Targeted_B_Plus_Tree, Search_Key);
         if (Targeted_B_Plus_Tree.Current_Leaf = null) then
            raise Key_Does_Not_Exist_In_This_B_Plus_Tree;
         else
            return Targeted_B_Plus_Tree.Current_Leaf.Item_Container;
         end if;
      end if;
   end Get_Item_Container;
   ----------------------------------------------------------------------------------------------------------------------------
   function A_Preceding_Key_Exists (Targeted_B_Plus_Tree : in B_Plus_Tree;
                                    Search_Key           : in Key_Type     ) return Boolean is

   begin   -- function A_Preceding_Key_Exists
      Targeted_B_Plus_Tree.Current_Leaf := Get_Leaf_Node (Targeted_B_Plus_Tree, Search_Key);
      return ( not Empty (Targeted_B_Plus_Tree)
                 and then (Targeted_B_Plus_Tree.Current_Leaf /= null) 
                 and then (Targeted_B_Plus_Tree.Current_Leaf.Preceding_Leaf /= null) );
   end A_Preceding_Key_Exists;
   ----------------------------------------------------------------------------------------------------------------------------
   function Get_Preceding_Key (Targeted_B_Plus_Tree : in B_Plus_Tree;
                               Search_Key           : in Key_Type     ) return Key_Type is

      -- Raises Key_Does_Not_Exist_In_This_B_Plus_Tree 
      --   or No_Preceding_Key_Exists_In_This_B_Plus_Tree
      --   or No_Keys_Currently_Exist_In_This_Empty_B_Plus_Tree, when appropriate...

      Temp : Key_Type;

   begin   -- function Get_Preceding_Key
      if not Empty (Targeted_B_Plus_Tree) then
         Targeted_B_Plus_Tree.Current_Leaf := Get_Leaf_Node (Targeted_B_Plus_Tree, Search_Key);
         if (Targeted_B_Plus_Tree.Current_Leaf /= null) 
          and then (Targeted_B_Plus_Tree.Current_Leaf.Preceding_Leaf /= null) then
            return Targeted_B_Plus_Tree.Current_Leaf.Preceding_Leaf.Key_Value.all;
         elsif (Targeted_B_Plus_Tree.Current_Leaf.Preceding_Leaf = null) then
            raise No_Preceding_Key_Exists_In_This_B_Plus_Tree;
         elsif (Targeted_B_Plus_Tree.Current_Leaf = null) then
            raise Key_Does_Not_Exist_In_This_B_Plus_Tree;
         end if;
      else
         raise No_Keys_Currently_Exist_In_This_Empty_B_Plus_Tree;
      end if;  
   end Get_Preceding_Key;
   ----------------------------------------------------------------------------------------------------------------------------
   function A_Following_Key_Exists  (Targeted_B_Plus_Tree : in B_Plus_Tree;
                                     Search_Key           : in Key_Type     ) return Boolean is

   begin   -- function A_Following_Key_Exists
      Targeted_B_Plus_Tree.Current_Leaf := Get_Leaf_Node (Targeted_B_Plus_Tree, Search_Key);
      return ( not Empty (Targeted_B_Plus_Tree)
                 and then (Targeted_B_Plus_Tree.Current_Leaf /= null) 
                 and then (Targeted_B_Plus_Tree.Current_Leaf.Following_Leaf /= null) );
   end A_Following_Key_Exists;
   ----------------------------------------------------------------------------------------------------------------------------
   function Get_Following_Key  (Targeted_B_Plus_Tree : in B_Plus_Tree;
                                Search_Key           : in Key_Type     ) return Key_Type is

      -- Raises Key_Does_Not_Exist_In_This_B_Plus_Tree 
      --   or No_Following_Key_Exists_In_This_B_Plus_Tree 
      --   or No_Keys_Currently_Exist_In_This_Empty_B_Plus_Tree, when appropriate...

   begin   -- function Get_Following_Key
      if not Empty (Targeted_B_Plus_Tree) then
         Targeted_B_Plus_Tree.Current_Leaf := Get_Leaf_Node (Targeted_B_Plus_Tree, Search_Key);
         if (Targeted_B_Plus_Tree.Current_Leaf /= null) 
          and then (Targeted_B_Plus_Tree.Current_Leaf.Following_Leaf /= null) then
            return Targeted_B_Plus_Tree.Current_Leaf.Following_Leaf.Key_Value.all;
         elsif (Targeted_B_Plus_Tree.Current_Leaf.Following_Leaf = null) then
            raise No_Following_Key_Exists_In_This_B_Plus_Tree;
         elsif (Targeted_B_Plus_Tree.Current_Leaf = null) then
            raise Key_Does_Not_Exist_In_This_B_Plus_Tree;
         end if;
      else
         raise No_Keys_Currently_Exist_In_This_Empty_B_Plus_Tree;
      end if;  
   end Get_Following_Key;
   ----------------------------------------------------------------------------------------------------------------------------
   procedure Assign (To_B_Plus_Tree   : in out B_Plus_Tree;
                     From_B_Plus_Tree : in     B_Plus_Tree) is

      Last_Leaf : Internal_Node_Pointer;

      procedure Assign (To_Internal_Node       : in out Internal_Node_Pointer;
                        Minimum_Key_In_Subtree :    out Key_Pointer;
                        From_Internal_Node     : in     Internal_Node_Pointer) is

      begin   -- procedure Assign
         To_Internal_Node := new Internal_Node (Index_Node => From_Internal_Node.Index_Node);
         if From_Internal_Node.Index_Node then
            To_Internal_Node.Maximum_Subtree_Index := From_Internal_Node.Maximum_Subtree_Index;
            Assign (To_Internal_Node.Subtree (0), Minimum_Key_In_Subtree, From_Internal_Node.Subtree (0));
            for Subtree_Number in 1..From_Internal_Node.Maximum_Subtree_Index loop
               Assign (To_Internal_Node.Subtree (Subtree_Number),
                          To_Internal_Node.Key ( Keys'Val ( Subtrees'Pos (Subtree_Number) ) ),
                             From_Internal_Node.Subtree (Subtree_Number));
            end loop;
         else
            To_Internal_Node.Preceding_Leaf := Last_Leaf;
            Last_Leaf.Following_Leaf := To_Internal_Node;
            To_Internal_Node.Key_Value := new Key_Type;
            Assign (To_Internal_Node.Key_Value.all, From_Internal_Node.Key_Value.all);
            Assign (To_Internal_Node.Item_Container, From_Internal_Node.Item_Container);
            Last_Leaf := To_Internal_Node;
         end if;
      end Assign;

   begin   -- procedure Assign
      Destroy (To_B_Plus_Tree);
      if (From_B_Plus_Tree /= null) then
         To_B_Plus_Tree := new B_Plus_Tree_Descriptor;
         Assign (To_B_Plus_Tree.Root_Node, To_B_Plus_Tree.Minimum_Key, From_B_Plus_Tree.Root_Node);
         To_B_Plus_Tree.Current_Leaf := Get_Leaf_Node (To_B_Plus_Tree, From_B_Plus_Tree.Current_Leaf.Key_Value.all);
         To_B_Plus_Tree.Number_Of_Keys_Stored := From_B_Plus_Tree.Number_Of_Keys_Stored;
      end if;
   end Assign;
   ----------------------------------------------------------------------------------------------------------------------------


end Generic_B_Plus_Tree;

