------------------------------------------------------------------------
--
-- Indexed sequential files in Ada: a didactical example
--
-- See also: Ada-Belgium Newsletter, Vol.4, November 1996
--           Proceedings 1996 Ada-Belgium Seminar
--           "Ada: the solution to integrate technologies"
--           Brussels, November 22, 1996
--
------------------------------------------------------------------------
--
-- This file contains three Ada packages
--     - the specification of "Aux_io_exceptions".
--     - the specification of "Indexed_io".
--     - the body of "Indexed_io".
--
-- For the Gnat-compiler, the three packages should be put in three
-- different files, named respectively "Auxioexc.ads", "Indexeio.ads"
-- and "Indexeio.adb".
-- The files
--     indexeio.ads
--     indexeio.adb
--     auxioexc.ads
-- should be copied in the directory C:\DJGPP\ADAINC
--
-- After compilation, the files
--     indexeio.o
--     auxioexc.o
--     auxioexc.ali
-- should be put in the directory C:\DJGPP\LIB\ADALIB
--
-- Information concerning the invokation of the compiler or the linker
-- can be found in the Ada-Belgium repository.
--
------------------------------------------------------------------------
--
-- Marc A Gobin
-- Bleukeveldlaan 7
-- 3080 Tervuren
-- Tel : +32(2)767.69.52
-- Fax : +32(2)767.18.90
-- E-mail : Marc.Gobin@info.rma.ac.be
--
------------------------------------------------------------------------

with io_exceptions, aux_io_exceptions, Ada.Direct_io ;
generic
   type element_type is private ;

package indexed_io is
   type file_type is limited private ;
   type file_mode is (in_file, inout_file, out_file) ;
   type relation_type is (greater_equal, equal, greater) ;
   procedure create (file : in out file_type ;
		     mode : in file_mode := inout_file ;
		     name : in string := "" ;
		     form : in string) ;
   procedure open (file : in out file_type ;
		   mode : in file_mode ;
		   name : in string ;
		   form : in string := "") ;
   procedure close (file : in out file_type) ;
   procedure delete (file : in out file_type) ;
   procedure reset (file : in out file_type ;
		    mode : in file_mode ;
		    key_number : in integer := 0) ;
   procedure reset (file : in out file_type ; key_number : in integer := 0) ;
   function mode (file : file_type) return file_mode ;
   function name (file : file_type) return string ;
   function form (file : file_type) return string ;
   function is_open (file : file_type) return boolean ;
   procedure read (file : in out file_type ; item : out element_type) ;
   generic
      type key_type is private ;
      default_key_number : integer := 0 ;
   procedure read_by_key (file : in out file_type ;
			  item : out element_type ;
			  key : in key_type ;
			  key_number : in integer := default_key_number ;
			  relation : in relation_type := equal) ;
   procedure write (file : in out file_type ; item : in element_type) ;
   procedure update (file : in out file_type ; item : in element_type) ;
   procedure delete_element (file : in out file_type) ;
   function end_of_file (file : file_type) return boolean ;
   status_error : exception renames io_exceptions.status_error ;
   mode_error : exception renames io_exceptions.mode_error ;
   name_error : exception renames io_exceptions.name_error ;
   use_error : exception renames io_exceptions.use_error ;
   device_error : exception renames io_exceptions.device_error ;
   end_error : exception renames io_exceptions.end_error ;
   data_error : exception renames io_exceptions.data_error ;
   key_error : exception renames aux_io_exceptions.key_error ;
   existence_error : exception renames aux_io_exceptions.existence_error ;
private
   subtype block is string (1..512) ;
   package dirio is new Ada.Direct_io (block) ;
   type file_tab is array (0..9) of dirio.file_type ;
   type key_prop is array (0..9) of integer ;
   type block_nr_typ is array (1..20) of integer ;
   type acc_block is access block ;
   type block_acc_typ is array (1..20) of acc_block ;
   type file_type is record
      data_file : dirio.file_type ;
      key_file : file_tab ;
      name : string (1..52) ;
      namlen, punt, size, formlen, num_keys, cur_key_no,
              num_rec, data_recup : integer ;
      mode : file_mode ;
      form : block ;
      key_length, key_position, key_levels, key_recup : key_prop ;
      block_nr, char_nr : block_nr_typ ;
      block_acc : block_acc_typ ;
      init : boolean := true ;
      opened : boolean := false ;
   end record ;
end indexed_io ;

