with Month_Declaration;

package body Date_Format_Template is

   procedure Insert_Date_Token (Into            : out String;
                                Using_Token     : in String;
                                Insertion_Point : in out Natural);

   function Is_Date_Delimiter
       (From : in Date_Delimiter) return Character;

   function Is_Valid_Date (M : in Month;
                           D : in Day;
                           Y : in Year) return Boolean;

   pragma INLINE (Is_Date_Delimiter, Is_Valid_Date);


   procedure Insert_Date_Token (Into            : out String;
                                Using_Token     : in String;
                                Insertion_Point : in out Natural) is
      Length : constant Natural := Using_Token'LENGTH;
   begin
      Into (Insertion_Point - Length + 1 .. Insertion_Point) :=
         Using_Token;
      Insertion_Point := Insertion_Point - Length;
   end Insert_Date_Token;


   function Is_Date_Delimiter
       (From : in Date_Delimiter) return Character is
      Delimiter : Character;
   begin
      case From is
         when Slash_Delimiter =>
            Delimiter := '/';
         when Hyphen_Delimiter =>
            Delimiter := '-';
         when Dot_Delimiter =>
            Delimiter := '.';
      end case;
      return Delimiter;
   end Is_Date_Delimiter;


   function Is_Valid_Date (M : in Month;
                           D : in Day;
                           Y : in Year) return Boolean is
   begin
      return not (M not in 1 .. 12 or else D not in 1 .. 31 or else
             Y < 0);
   end Is_Valid_Date;


   function Is_Long_Date (From_Month       : in Month;
                          From_Day         : in Day;
                          From_Year        : in Year;
                          Using_Date_Style : in Date_Style :=
      Month_Preceding) return Long_Date is
   begin
      if Is_Valid_Date (From_Month, From_Day, From_Year) then
         null;
      else
         raise Operation_Failure;
      end if;
   Format_Long_Date:
      declare
         M : constant String :=
            Month_Declaration.Month'IMAGE (
               Month_Declaration.Month'VAL (From_Month - 1));
         D : constant String := Day'IMAGE (From_Day);
         Y : constant String := Year'IMAGE (From_Year);

         Y_Size          : constant Positive := Y'LENGTH;
         String_Date     : Long_Date;
         Insertion_Point : Natural := String_Date'LAST;
      begin
         case Using_Date_Style is
            when Month_Preceding =>
               Insert_Date_Token (String_Date, Y, Insertion_Point);
               Insert_Date_Token (String_Date, ",", Insertion_Point);
               Insert_Date_Token (String_Date, D, Insertion_Point);
               Insert_Date_Token (String_Date, M, Insertion_Point);
            when Day_Preceding =>
               Insert_Date_Token (String_Date, Y, Insertion_Point);
               Insert_Date_Token (String_Date, M, Insertion_Point);
               Insert_Date_Token (String_Date, " ", Insertion_Point);
               Insert_Date_Token (String_Date, D (2 .. D'LAST),
                                  Insertion_Point);
            when Year_Preceding =>
               Insert_Date_Token (String_Date, D, Insertion_Point);
               Insert_Date_Token (String_Date, M, Insertion_Point);
               Insert_Date_Token (String_Date, " ", Insertion_Point);
               Insert_Date_Token (String_Date, Y (2 .. Y'LAST),
                                  Insertion_Point);
         end case;
         if Insertion_Point >= 1 then
            Insert_Date_Token (String_Date,
                               String'(1 .. Insertion_Point => ' '),
                               Insertion_Point);
         end if;
         return String_Date;
      end Format_Long_Date;
   end Is_Long_Date;


   function Is_Short_Date
       (From_Month           : in Month;
        From_Day             : in Day;
        From_Year            : in Year;
        Using_Date_Style     : in Date_Style     := Month_Preceding;
        Using_Date_Delimiter : in Date_Delimiter := Slash_Delimiter)
       return Short_Date is
   begin
      if Is_Valid_Date (From_Month, From_Day, From_Year) then
         null;
      else
         raise Operation_Failure;
      end if;
   Format_Short_Date:
      declare
         M : constant String := Month'IMAGE (From_Month);
         D : constant String := Day'IMAGE (From_Day);
         Y : constant String := Year'IMAGE (From_Year);

         Y_Size : constant Positive := Y'LENGTH;

         Delim           : String (1 .. 1);
         String_Date     : Short_Date;
         Insertion_Point : Natural := String_Date'LAST;
      begin
         Delim (1) := Is_Date_Delimiter (Using_Date_Delimiter);
         case Using_Date_Style is
            when Month_Preceding =>
               Insert_Date_Token (String_Date,
                                  Y (Y_Size - 1 .. Y_Size),
                                  Insertion_Point);
               Insert_Date_Token
                   (String_Date, Delim, Insertion_Point);
               Insert_Date_Token (String_Date, D (2 .. D'LAST),
                                  Insertion_Point);
               Insert_Date_Token
                   (String_Date, Delim, Insertion_Point);
               Insert_Date_Token (String_Date, M (2 .. M'LAST),
                                  Insertion_Point);
            when Day_Preceding =>
               Insert_Date_Token (String_Date,
                                  Y (Y_Size - 1 .. Y_Size),
                                  Insertion_Point);
               Insert_Date_Token
                   (String_Date, Delim, Insertion_Point);
               Insert_Date_Token (String_Date, M (2 .. M'LAST),
                                  Insertion_Point);
               Insert_Date_Token
                   (String_Date, Delim, Insertion_Point);
               Insert_Date_Token (String_Date, D (2 .. D'LAST),
                                  Insertion_Point);
            when Year_Preceding =>
               Insert_Date_Token (String_Date, D (2 .. D'LAST),
                                  Insertion_Point);
               Insert_Date_Token
                   (String_Date, Delim, Insertion_Point);
               Insert_Date_Token (String_Date, M (2 .. M'LAST),
                                  Insertion_Point);
               Insert_Date_Token
                   (String_Date, Delim, Insertion_Point);
               Insert_Date_Token (String_Date,
                                  Y (Y_Size - 1 .. Y_Size),
                                  Insertion_Point);
         end case;
         if Insertion_Point >= 1 then
            Insert_Date_Token (String_Date,
                               String'(1 .. Insertion_Point => ' '),
                               Insertion_Point);
         end if;
         return String_Date;
      end Format_Short_Date;
   end Is_Short_Date;

end Date_Format_Template;
