with Dbms_Date;

package Parts_Inventory_Database is

   type Part_Number is new String (1 .. 8);
   type Customer_Number is new String (1 .. 8);
   type Quantity is new Long_Integer;

   type Data_Record_Description is
      record
         The_Part_Number     : Part_Number;
         The_Customer_Number : Customer_Number;
         The_Quantity        : Quantity;
         The_Date            : Dbms_Date.Date_Declaration.Date;
      end record;

   procedure Open;
   procedure Close;

   procedure Find_Inventory_Record
      (For_Part_Number : in Part_Number;
       For_Customer_Number : in Customer_Number);

   procedure Append (Using : in Data_Record_Description);

   function Is_Current_Record return Data_Record_Description;

   function Is_The_Part_Number
      return Part_Number;
   function Is_The_Customer_Number
      return Customer_Number;

   function Is_The_Quantity
      return Quantity;
   function Is_The_Date
      return Dbms_Date.Date_Declaration.Date;


   function Match_All return Boolean;


   generic
      with function Is_Match return Boolean is Match_All;

   package Customers_To_Part_Query_Template is
   
      procedure Initialize_Query (For_Part : in Part_Number);
      procedure Find_Next;
   
   end Customers_To_Part_Query_Template;


   generic
      with function Is_Match return Boolean is Match_All;

   package Sequential_Search_Template is

      procedure Find_Next;

   end Sequential_Search_Template;


   End_Of_Data : Exception;

end Parts_Inventory_Database;
