
-------------------------------------------------------------------------------
--                                                                           --
--  Component
--
--  $Id: read_file.ads,v 1.5 1998/12/30 19:34:26 obry Exp $
--
--  Author    : Pascal Obry
--  e-mail    : pascal_obry@csi.com
--
--  Machine/System Compiled/Run on : SPARC SUN-OS, Windows NT - GNAT
--
--                                                                           --
-------------------------------------------------------------------------------

--
--
--  Read Formatted Files
--  --------------------
--
--  Each line is a set of field. Fields are separated by ' ' or Tab or
--  user defined separators.
--
--  The file name "-" is used for standard input.
--

generic

    Max_Field          : Positive :=    30;
    Max_Line_Lenght    : Positive := 1_000;

package Read_File is

   subtype Field_Range is Positive range 1 .. Max_Field;

   procedure Open (Name : in String);
   procedure Close;
   procedure Set_Separators (To : in String);
   --  define a set of separators

   procedure Fetch;
   --  read next line
   function End_Of_File
     return Boolean;

   function Is_Empty_Line
     return Boolean;

   function Number_Of_Fields
     return Natural;
   --  number of field for the current line

   function Line_Number
     return Natural;

   --  to get the separator that has been used to cut the field.
   function Separator_For_Field (N : in Field_Range)
                                 return Character;

   --  to get the value of a field as a String, Integer, Float
   function Field (N : in Field_Range)
      return String;
   function Field (N : in Field_Range)
      return Integer;
   function Field (N : in Field_Range)
      return Float;

   --  to get the value of a field as a user defined type.
   generic
      type T is (<>);
   function Field_Of_Type (N : in Field_Range)
      return T;

end Read_File;
